#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_ami_info
version_added: 1.0.0
short_description: Gather information about ec2 AMIs
description:
  - Gather information about ec2 AMIs
author:
  - Prasad Katti (@prasadkatti)
options:
  image_ids:
    description: One or more image IDs.
    aliases: [image_id]
    type: list
    elements: str
    default: []
  filters:
    description:
      - A dict of filters to apply. Each dict item consists of a filter key and a filter value.
      - See U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeImages.html) for possible filters.
      - Filter names and values are case sensitive.
    type: dict
    default: {}
  owners:
    description:
      - Filter the images by the owner. Valid options are an AWS account ID, self,
        or an AWS owner alias ( amazon | aws-marketplace | microsoft ).
    aliases: [owner]
    type: list
    elements: str
    default: []
  executable_users:
    description:
      - Filter images by users with explicit launch permissions. Valid options are an AWS account ID, self, or all (public AMIs).
    aliases: [executable_user]
    type: list
    elements: str
    default: []
  describe_image_attributes:
    description:
      - Describe attributes (like launchPermission) of the images found.
    default: false
    type: bool

extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: gather information about an AMI using ami-id
  amazon.aws.ec2_ami_info:
    image_ids: ami-5b488823

- name: gather information about all AMIs with tag key Name and value webapp
  amazon.aws.ec2_ami_info:
    filters:
      "tag:Name": webapp

- name: gather information about an AMI with 'AMI Name' equal to foobar
  amazon.aws.ec2_ami_info:
    filters:
      name: foobar

- name: gather information about Ubuntu 17.04 AMIs published by Canonical (099720109477)
  amazon.aws.ec2_ami_info:
    owners: 099720109477
    filters:
      name: "ubuntu/images/ubuntu-zesty-17.04-*"
"""

RETURN = r"""
images:
  description: A list of images.
  returned: always
  type: list
  elements: dict
  contains:
    architecture:
      description: The architecture of the image.
      returned: always
      type: str
      sample: x86_64
    block_device_mappings:
      description: Any block device mapping entries.
      returned: always
      type: list
      elements: dict
      contains:
        device_name:
          description: The device name exposed to the instance.
          returned: always
          type: str
          sample: /dev/sda1
        ebs:
          description: EBS volumes
          returned: always
          type: complex
    creation_date:
      description: The date and time the image was created.
      returned: always
      type: str
      sample: '2017-10-16T19:22:13.000Z'
    description:
      description: The description of the AMI.
      returned: always
      type: str
      sample: ''
    ena_support:
      description: Whether enhanced networking with ENA is enabled.
      returned: always
      type: bool
      sample: true
    hypervisor:
      description: The hypervisor type of the image.
      returned: always
      type: str
      sample: xen
    image_id:
      description: The ID of the AMI.
      returned: always
      type: str
      sample: ami-5b466623
    image_location:
      description: The location of the AMI.
      returned: always
      type: str
      sample: 123456789012/Webapp
    image_type:
      description: The type of image.
      returned: always
      type: str
      sample: machine
    launch_permissions:
      description: A List of AWS accounts may launch the AMI.
      returned: When image is owned by calling account and I(describe_image_attributes=true).
      type: list
      elements: dict
      contains:
        group:
            description: A value of 'all' means the AMI is public.
            type: str
        user_id:
            description: An AWS account ID with permissions to launch the AMI.
            type: str
      sample: [{"group": "all"}, {"user_id": "123456789012"}]
    name:
      description: The name of the AMI that was provided during image creation.
      returned: always
      type: str
      sample: Webapp
    owner_id:
      description: The AWS account ID of the image owner.
      returned: always
      type: str
      sample: '123456789012'
    public:
      description: Whether the image has public launch permissions.
      returned: always
      type: bool
      sample: true
    root_device_name:
      description: The device name of the root device.
      returned: always
      type: str
      sample: /dev/sda1
    root_device_type:
      description: The type of root device used by the AMI.
      returned: always
      type: str
      sample: ebs
    sriov_net_support:
      description: Whether enhanced networking is enabled.
      returned: always
      type: str
      sample: simple
    state:
      description: The current state of the AMI.
      returned: always
      type: str
      sample: available
    tags:
      description: Any tags assigned to the image.
      returned: always
      type: dict
    virtualization_type:
      description: The type of virtualization of the AMI.
      returned: always
      type: str
      sample: hvm
"""

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list


class AmiInfoFailure(Exception):
    def __init__(self, original_e, user_message):
        self.original_e = original_e
        self.user_message = user_message
        super().__init__(self)


def build_request_args(executable_users, filters, image_ids, owners):
    request_args = {
        "ExecutableUsers": [str(user) for user in executable_users],
        "ImageIds": [str(image_id) for image_id in image_ids],
    }

    # describe_images is *very* slow if you pass the `Owners`
    # param (unless it's self), for some reason.
    # Converting the owners to filters and removing from the
    # owners param greatly speeds things up.
    # Implementation based on aioue's suggestion in #24886
    for owner in owners:
        if owner.isdigit():
            if "owner-id" not in filters:
                filters["owner-id"] = list()
            filters["owner-id"].append(owner)
        elif owner == "self":
            # self not a valid owner-alias filter (https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeImages.html)
            request_args["Owners"] = [str(owner)]
        else:
            if "owner-alias" not in filters:
                filters["owner-alias"] = list()
            filters["owner-alias"].append(owner)

    request_args["Filters"] = ansible_dict_to_boto3_filter_list(filters)

    request_args = {k: v for k, v in request_args.items() if v}

    return request_args


def get_images(ec2_client, request_args):
    try:
        images = ec2_client.describe_images(aws_retry=True, **request_args)
    except (ClientError, BotoCoreError) as err:
        raise AmiInfoFailure(err, "error describing images")
    return images


def get_image_attribute(ec2_client, image_id):
    try:
        launch_permissions = ec2_client.describe_image_attribute(
            aws_retry=True, Attribute="launchPermission", ImageId=image_id
        )
    except (ClientError, BotoCoreError) as err:
        raise AmiInfoFailure(err, "error describing image attribute")
    return launch_permissions


def list_ec2_images(ec2_client, module, request_args):
    images = get_images(ec2_client, request_args)["Images"]
    images = [camel_dict_to_snake_dict(image) for image in images]

    for image in images:
        try:
            image_id = image["image_id"]
            image["tags"] = boto3_tag_list_to_ansible_dict(image.get("tags", []))
            if module.params.get("describe_image_attributes"):
                launch_permissions = get_image_attribute(ec2_client, image_id).get("LaunchPermissions", [])
                image["launch_permissions"] = [camel_dict_to_snake_dict(perm) for perm in launch_permissions]
        except is_boto3_error_code("AuthFailure"):
            # describing launch permissions of images owned by others is not permitted, but shouldn't cause failures
            pass
        except (ClientError, BotoCoreError) as err:  # pylint: disable=duplicate-except
            raise AmiInfoFailure(err, "Failed to describe AMI")

    images.sort(key=lambda e: e.get("creation_date", ""))  # it may be possible that creation_date does not always exist

    return images


def main():
    argument_spec = dict(
        describe_image_attributes=dict(default=False, type="bool"),
        executable_users=dict(default=[], type="list", elements="str", aliases=["executable_user"]),
        filters=dict(default={}, type="dict"),
        image_ids=dict(default=[], type="list", elements="str", aliases=["image_id"]),
        owners=dict(default=[], type="list", elements="str", aliases=["owner"]),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    ec2_client = module.client("ec2", retry_decorator=AWSRetry.jittered_backoff())

    request_args = build_request_args(
        executable_users=module.params["executable_users"],
        filters=module.params["filters"],
        image_ids=module.params["image_ids"],
        owners=module.params["owners"],
    )

    images = list_ec2_images(ec2_client, module, request_args)

    module.exit_json(images=images)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Red Hat Inc.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = '''
---
module: rhel_rpm_ostree
version_added: 1.5.0
short_description: Ensure packages exist in a RHEL for Edge rpm-ostree based system
description:
  - Compatibility layer for using the "package" module for RHEL for Edge systems utilizing the RHEL System Roles.
author:
  - Adam Miller (@maxamillion)
requirements:
  - rpm-ostree
options:
  name:
    description:
      - A package name or package specifier with version, like C(name-1.0).
      - Comparison operators for package version are valid here C(>), C(<), C(>=), C(<=). Example - C(name>=1.0)
      - If a previous version is specified, the task also needs to turn C(allow_downgrade) on.
        See the C(allow_downgrade) documentation for caveats with downgrading packages.
      - When using state=latest, this can be C('*') which means run C(yum -y update).
      - You can also pass a url or a local path to a rpm file (using state=present).
        To operate on several packages this can accept a comma separated string of packages or (as of 2.0) a list of packages.
    aliases: [ pkg ]
    type: list
    elements: str
    default: []
  state:
    description:
      - Whether to install (C(present) or C(installed), C(latest)), or remove (C(absent) or C(removed)) a package.
      - C(present) and C(installed) will simply ensure that a desired package is installed.
      - C(latest) will update the specified package if it's not of the latest available version.
      - C(absent) and C(removed) will remove the specified package.
      - Default is C(None), however in effect the default action is C(present) unless the C(autoremove) option is
        enabled for this module, then C(absent) is inferred.
    type: str
    choices: [ absent, installed, latest, present, removed ]
notes:
  - This module does not support installing or removing packages to/from an overlay as this is not supported
    by RHEL for Edge, packages needed should be defined in the osbuild Blueprint and provided to Image Builder
    at build time. This module exists only for C(package) module compatibility.
'''

EXAMPLES = '''
- name: Ensure htop and ansible are installed on rpm-ostree based RHEL
  ansible.posix.rhel_rpm_ostree:
    name:
      - htop
      - ansible
    state: present
'''

RETURN = """
msg:
    description: status of rpm transaction
    returned: always
    type: str
    sample: "No changes made."
"""

import os
import traceback

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils._text import to_text


def locally_installed(module, pkgname):
    (rc, out, err) = module.run_command('{0} -q {1}'.format(module.get_bin_path("rpm"), pkgname).split())
    return (rc == 0)


def rpm_ostree_transaction(module):
    pkgs = []

    if module.params['state'] in ['present', 'installed', 'latest']:
        for pkg in module.params['name']:
            if not locally_installed(module, pkg):
                pkgs.append(pkg)
    elif module.params['state'] in ['absent', 'removed']:
        for pkg in module.params['name']:
            if locally_installed(module, pkg):
                pkgs.append(pkg)

    if not pkgs:
        module.exit_json(msg="No changes made.")
    else:
        if module.params['state'] in ['present', 'installed', 'latest']:
            module.fail_json(msg="The following packages are absent in the currently booted rpm-ostree commit: %s" ' '.join(pkgs))
        else:
            module.fail_json(msg="The following packages are present in the currently booted rpm-ostree commit: %s" ' '.join(pkgs))


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(type='list', elements='str', aliases=['pkg'], default=[]),
            state=dict(type='str', default=None, choices=['absent', 'installed', 'latest', 'present', 'removed']),
        ),
    )

    # Verify that the platform is an rpm-ostree based system
    if not os.path.exists("/run/ostree-booted"):
        module.fail_json(msg="Module rpm_ostree is only applicable for rpm-ostree based systems.")

    try:
        rpm_ostree_transaction(module)
    except Exception as e:
        module.fail_json(msg=to_text(e), exception=traceback.format_exc())


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: platform_nodes_configuration_summary_info
short_description: Information module for Platform Nodes Configuration Summary
description:
- Get all Platform Nodes Configuration Summary.
- >
   Provides details about the current Cisco DNA Center node configuration, such as API version, node name, NTP
   server, intracluster link, LACP mode, network static routes, DNS server, subnet mask, host IP, default gateway,
   and interface information.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Platform Configuration CiscoDNACenterNodesConfigurationSummary
  description: Complete reference of the CiscoDNACenterNodesConfigurationSummary API.
  link: https://developer.cisco.com/docs/dna-center/#!cisco-dna-center-nodes-configuration-summary
notes:
  - SDK Method used are
    platform_configuration.PlatformConfiguration.nodes_configuration_summary,

  - Paths used are
    get /dna/intent/api/v1/nodes-config,

"""

EXAMPLES = r"""
- name: Get all Platform Nodes Configuration Summary
  cisco.dnac.platform_nodes_configuration_summary_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
  register: result

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "nodes": [
          {
            "ntp": {
              "servers": [
                "string"
              ]
            },
            "network": [
              {
                "intra_cluster_link": true,
                "lacp_mode": true,
                "inet": {
                  "routes": [
                    {}
                  ],
                  "gateway": "string",
                  "dns_servers": [
                    {}
                  ],
                  "netmask": "string",
                  "host_ip": "string"
                },
                "interface": "string",
                "inet6": {
                  "host_ip": "string",
                  "netmask": "string"
                },
                "lacp_supported": true,
                "slave": [
                  "string"
                ]
              }
            ],
            "proxy": {
              "https_proxy": "string",
              "no_proxy": [
                "string"
              ],
              "https_proxy_username": "string",
              "http_proxy": "string",
              "https_proxy_password": "string"
            },
            "platform": {
              "vendor": "string",
              "product": "string",
              "serial": "string"
            },
            "id": "string",
            "name": "string"
          }
        ]
      }
    }
"""



# -*- coding: utf-8 -*-
# This code is part of the Ansible collection community.docker, but is an independent component.
# This particular file, and this file only, is based on the Docker SDK for Python (https://github.com/docker/docker-py/)
#
# Copyright (c) 2016-2022 Docker, Inc.
#
# It is licensed under the Apache 2.0 license (see LICENSES/Apache-2.0.txt in this collection)
# SPDX-License-Identifier: Apache-2.0

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import os
import sys

from ansible.module_utils.six import PY2

if PY2:
    from distutils.spawn import find_executable as which
else:
    from shutil import which


def find_executable(executable, path=None):
    """
    As distutils.spawn.find_executable, but on Windows, look up
    every extension declared in PATHEXT instead of just `.exe`
    """
    if not PY2:
        # shutil.which() already uses PATHEXT on Windows, so on
        # Python 3 we can simply use shutil.which() in all cases.
        # (https://github.com/docker/docker-py/commit/42789818bed5d86b487a030e2e60b02bf0cfa284)
        return which(executable, path=path)

    if sys.platform != 'win32':
        return which(executable, path)

    if path is None:
        path = os.environ['PATH']

    paths = path.split(os.pathsep)
    extensions = os.environ.get('PATHEXT', '.exe').split(os.pathsep)
    base, ext = os.path.splitext(executable)

    if not os.path.isfile(executable):
        for p in paths:
            for ext in extensions:
                f = os.path.join(p, base + ext)
                if os.path.isfile(f):
                    return f
        return None
    else:
        return executable


def create_environment_dict(overrides):
    """
    Create and return a copy of os.environ with the specified overrides
    """
    result = os.environ.copy()
    result.update(overrides or {})
    return result



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016, Yanis Guenane <yanis+ansible@guenane.org>
# Copyright (c) 2020, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type


import abc
import base64
import traceback

from ansible.module_utils import six
from ansible.module_utils.basic import missing_required_lib
from ansible.module_utils.common.text.converters import to_bytes

from ansible_collections.community.crypto.plugins.module_utils.version import LooseVersion

from ansible_collections.community.crypto.plugins.module_utils.crypto.basic import (
    CRYPTOGRAPHY_HAS_X25519,
    CRYPTOGRAPHY_HAS_X25519_FULL,
    CRYPTOGRAPHY_HAS_X448,
    CRYPTOGRAPHY_HAS_ED25519,
    CRYPTOGRAPHY_HAS_ED448,
    OpenSSLObjectError,
)

from ansible_collections.community.crypto.plugins.module_utils.crypto.support import (
    get_fingerprint_of_privatekey,
)

from ansible_collections.community.crypto.plugins.module_utils.crypto.pem import (
    identify_private_key_format,
)

from ansible_collections.community.crypto.plugins.module_utils.crypto.module_backends.privatekey_info import (
    PrivateKeyConsistencyError,
    PrivateKeyParseError,
    get_privatekey_info,
)

from ansible_collections.community.crypto.plugins.module_utils.crypto.module_backends.common import ArgumentSpec


MINIMAL_CRYPTOGRAPHY_VERSION = '1.2.3'

CRYPTOGRAPHY_IMP_ERR = None
try:
    import cryptography
    import cryptography.exceptions
    import cryptography.hazmat.backends
    import cryptography.hazmat.primitives.serialization
    import cryptography.hazmat.primitives.asymmetric.rsa
    import cryptography.hazmat.primitives.asymmetric.dsa
    import cryptography.hazmat.primitives.asymmetric.ec
    import cryptography.hazmat.primitives.asymmetric.utils
    CRYPTOGRAPHY_VERSION = LooseVersion(cryptography.__version__)
except ImportError:
    CRYPTOGRAPHY_IMP_ERR = traceback.format_exc()
    CRYPTOGRAPHY_FOUND = False
else:
    CRYPTOGRAPHY_FOUND = True


class PrivateKeyError(OpenSSLObjectError):
    pass


# From the object called `module`, only the following properties are used:
#
#  - module.params[]
#  - module.warn(msg: str)
#  - module.fail_json(msg: str, **kwargs)


@six.add_metaclass(abc.ABCMeta)
class PrivateKeyBackend:
    def __init__(self, module, backend):
        self.module = module
        self.type = module.params['type']
        self.size = module.params['size']
        self.curve = module.params['curve']
        self.passphrase = module.params['passphrase']
        self.cipher = module.params['cipher']
        self.format = module.params['format']
        self.format_mismatch = module.params.get('format_mismatch', 'regenerate')
        self.regenerate = module.params.get('regenerate', 'full_idempotence')
        self.backend = backend

        self.private_key = None

        self.existing_private_key = None
        self.existing_private_key_bytes = None

        self.diff_before = self._get_info(None)
        self.diff_after = self._get_info(None)

    def _get_info(self, data):
        if data is None:
            return dict()
        result = dict(can_parse_key=False)
        try:
            result.update(get_privatekey_info(
                self.module, self.backend, data, passphrase=self.passphrase,
                return_private_key_data=False, prefer_one_fingerprint=True))
        except PrivateKeyConsistencyError as exc:
            result.update(exc.result)
        except PrivateKeyParseError as exc:
            result.update(exc.result)
        except Exception as exc:
            pass
        return result

    @abc.abstractmethod
    def generate_private_key(self):
        """(Re-)Generate private key."""
        pass

    def convert_private_key(self):
        """Convert existing private key (self.existing_private_key) to new private key (self.private_key).

        This is effectively a copy without active conversion. The conversion is done
        during load and store; get_private_key_data() uses the destination format to
        serialize the key.
        """
        self._ensure_existing_private_key_loaded()
        self.private_key = self.existing_private_key

    @abc.abstractmethod
    def get_private_key_data(self):
        """Return bytes for self.private_key."""
        pass

    def set_existing(self, privatekey_bytes):
        """Set existing private key bytes. None indicates that the key does not exist."""
        self.existing_private_key_bytes = privatekey_bytes
        self.diff_after = self.diff_before = self._get_info(self.existing_private_key_bytes)

    def has_existing(self):
        """Query whether an existing private key is/has been there."""
        return self.existing_private_key_bytes is not None

    @abc.abstractmethod
    def _check_passphrase(self):
        """Check whether provided passphrase matches, assuming self.existing_private_key_bytes has been populated."""
        pass

    @abc.abstractmethod
    def _ensure_existing_private_key_loaded(self):
        """Make sure that self.existing_private_key is populated from self.existing_private_key_bytes."""
        pass

    @abc.abstractmethod
    def _check_size_and_type(self):
        """Check whether provided size and type matches, assuming self.existing_private_key has been populated."""
        pass

    @abc.abstractmethod
    def _check_format(self):
        """Check whether the key file format, assuming self.existing_private_key and self.existing_private_key_bytes has been populated."""
        pass

    def needs_regeneration(self):
        """Check whether a regeneration is necessary."""
        if self.regenerate == 'always':
            return True
        if not self.has_existing():
            # key does not exist
            return True
        if not self._check_passphrase():
            if self.regenerate == 'full_idempotence':
                return True
            self.module.fail_json(msg='Unable to read the key. The key is protected with a another passphrase / no passphrase or broken.'
                                  ' Will not proceed. To force regeneration, call the module with `generate`'
                                  ' set to `full_idempotence` or `always`, or with `force=true`.')
        self._ensure_existing_private_key_loaded()
        if self.regenerate != 'never':
            if not self._check_size_and_type():
                if self.regenerate in ('partial_idempotence', 'full_idempotence'):
                    return True
                self.module.fail_json(msg='Key has wrong type and/or size.'
                                      ' Will not proceed. To force regeneration, call the module with `generate`'
                                      ' set to `partial_idempotence`, `full_idempotence` or `always`, or with `force=true`.')
        # During generation step, regenerate if format does not match and format_mismatch == 'regenerate'
        if self.format_mismatch == 'regenerate' and self.regenerate != 'never':
            if not self._check_format():
                if self.regenerate in ('partial_idempotence', 'full_idempotence'):
                    return True
                self.module.fail_json(msg='Key has wrong format.'
                                      ' Will not proceed. To force regeneration, call the module with `generate`'
                                      ' set to `partial_idempotence`, `full_idempotence` or `always`, or with `force=true`.'
                                      ' To convert the key, set `format_mismatch` to `convert`.')
        return False

    def needs_conversion(self):
        """Check whether a conversion is necessary. Must only be called if needs_regeneration() returned False."""
        # During conversion step, convert if format does not match and format_mismatch == 'convert'
        self._ensure_existing_private_key_loaded()
        return self.has_existing() and self.format_mismatch == 'convert' and not self._check_format()

    def _get_fingerprint(self):
        if self.private_key:
            return get_fingerprint_of_privatekey(self.private_key, backend=self.backend)
        try:
            self._ensure_existing_private_key_loaded()
        except Exception as dummy:
            # Ignore errors
            pass
        if self.existing_private_key:
            return get_fingerprint_of_privatekey(self.existing_private_key, backend=self.backend)

    def dump(self, include_key):
        """Serialize the object into a dictionary."""

        if not self.private_key:
            try:
                self._ensure_existing_private_key_loaded()
            except Exception as dummy:
                # Ignore errors
                pass
        result = {
            'type': self.type,
            'size': self.size,
            'fingerprint': self._get_fingerprint(),
        }
        if self.type == 'ECC':
            result['curve'] = self.curve
        # Get hold of private key bytes
        pk_bytes = self.existing_private_key_bytes
        if self.private_key is not None:
            pk_bytes = self.get_private_key_data()
        self.diff_after = self._get_info(pk_bytes)
        if include_key:
            # Store result
            if pk_bytes:
                if identify_private_key_format(pk_bytes) == 'raw':
                    result['privatekey'] = base64.b64encode(pk_bytes)
                else:
                    result['privatekey'] = pk_bytes.decode('utf-8')
            else:
                result['privatekey'] = None

        result['diff'] = dict(
            before=self.diff_before,
            after=self.diff_after,
        )
        return result


# Implementation with using cryptography
class PrivateKeyCryptographyBackend(PrivateKeyBackend):

    def _get_ec_class(self, ectype):
        ecclass = cryptography.hazmat.primitives.asymmetric.ec.__dict__.get(ectype)
        if ecclass is None:
            self.module.fail_json(msg='Your cryptography version does not support {0}'.format(ectype))
        return ecclass

    def _add_curve(self, name, ectype, deprecated=False):
        def create(size):
            ecclass = self._get_ec_class(ectype)
            return ecclass()

        def verify(privatekey):
            ecclass = self._get_ec_class(ectype)
            return isinstance(privatekey.private_numbers().public_numbers.curve, ecclass)

        self.curves[name] = {
            'create': create,
            'verify': verify,
            'deprecated': deprecated,
        }

    def __init__(self, module):
        super(PrivateKeyCryptographyBackend, self).__init__(module=module, backend='cryptography')

        self.curves = dict()
        self._add_curve('secp224r1', 'SECP224R1')
        self._add_curve('secp256k1', 'SECP256K1')
        self._add_curve('secp256r1', 'SECP256R1')
        self._add_curve('secp384r1', 'SECP384R1')
        self._add_curve('secp521r1', 'SECP521R1')
        self._add_curve('secp192r1', 'SECP192R1', deprecated=True)
        self._add_curve('sect163k1', 'SECT163K1', deprecated=True)
        self._add_curve('sect163r2', 'SECT163R2', deprecated=True)
        self._add_curve('sect233k1', 'SECT233K1', deprecated=True)
        self._add_curve('sect233r1', 'SECT233R1', deprecated=True)
        self._add_curve('sect283k1', 'SECT283K1', deprecated=True)
        self._add_curve('sect283r1', 'SECT283R1', deprecated=True)
        self._add_curve('sect409k1', 'SECT409K1', deprecated=True)
        self._add_curve('sect409r1', 'SECT409R1', deprecated=True)
        self._add_curve('sect571k1', 'SECT571K1', deprecated=True)
        self._add_curve('sect571r1', 'SECT571R1', deprecated=True)
        self._add_curve('brainpoolP256r1', 'BrainpoolP256R1', deprecated=True)
        self._add_curve('brainpoolP384r1', 'BrainpoolP384R1', deprecated=True)
        self._add_curve('brainpoolP512r1', 'BrainpoolP512R1', deprecated=True)

        self.cryptography_backend = cryptography.hazmat.backends.default_backend()

        if not CRYPTOGRAPHY_HAS_X25519 and self.type == 'X25519':
            self.module.fail_json(msg='Your cryptography version does not support X25519')
        if not CRYPTOGRAPHY_HAS_X25519_FULL and self.type == 'X25519':
            self.module.fail_json(msg='Your cryptography version does not support X25519 serialization')
        if not CRYPTOGRAPHY_HAS_X448 and self.type == 'X448':
            self.module.fail_json(msg='Your cryptography version does not support X448')
        if not CRYPTOGRAPHY_HAS_ED25519 and self.type == 'Ed25519':
            self.module.fail_json(msg='Your cryptography version does not support Ed25519')
        if not CRYPTOGRAPHY_HAS_ED448 and self.type == 'Ed448':
            self.module.fail_json(msg='Your cryptography version does not support Ed448')

    def _get_wanted_format(self):
        if self.format not in ('auto', 'auto_ignore'):
            return self.format
        if self.type in ('X25519', 'X448', 'Ed25519', 'Ed448'):
            return 'pkcs8'
        else:
            return 'pkcs1'

    def generate_private_key(self):
        """(Re-)Generate private key."""
        try:
            if self.type == 'RSA':
                self.private_key = cryptography.hazmat.primitives.asymmetric.rsa.generate_private_key(
                    public_exponent=65537,  # OpenSSL always uses this
                    key_size=self.size,
                    backend=self.cryptography_backend
                )
            if self.type == 'DSA':
                self.private_key = cryptography.hazmat.primitives.asymmetric.dsa.generate_private_key(
                    key_size=self.size,
                    backend=self.cryptography_backend
                )
            if CRYPTOGRAPHY_HAS_X25519_FULL and self.type == 'X25519':
                self.private_key = cryptography.hazmat.primitives.asymmetric.x25519.X25519PrivateKey.generate()
            if CRYPTOGRAPHY_HAS_X448 and self.type == 'X448':
                self.private_key = cryptography.hazmat.primitives.asymmetric.x448.X448PrivateKey.generate()
            if CRYPTOGRAPHY_HAS_ED25519 and self.type == 'Ed25519':
                self.private_key = cryptography.hazmat.primitives.asymmetric.ed25519.Ed25519PrivateKey.generate()
            if CRYPTOGRAPHY_HAS_ED448 and self.type == 'Ed448':
                self.private_key = cryptography.hazmat.primitives.asymmetric.ed448.Ed448PrivateKey.generate()
            if self.type == 'ECC' and self.curve in self.curves:
                if self.curves[self.curve]['deprecated']:
                    self.module.warn('Elliptic curves of type {0} should not be used for new keys!'.format(self.curve))
                self.private_key = cryptography.hazmat.primitives.asymmetric.ec.generate_private_key(
                    curve=self.curves[self.curve]['create'](self.size),
                    backend=self.cryptography_backend
                )
        except cryptography.exceptions.UnsupportedAlgorithm as dummy:
            self.module.fail_json(msg='Cryptography backend does not support the algorithm required for {0}'.format(self.type))

    def get_private_key_data(self):
        """Return bytes for self.private_key"""
        # Select export format and encoding
        try:
            export_format = self._get_wanted_format()
            export_encoding = cryptography.hazmat.primitives.serialization.Encoding.PEM
            if export_format == 'pkcs1':
                # "TraditionalOpenSSL" format is PKCS1
                export_format = cryptography.hazmat.primitives.serialization.PrivateFormat.TraditionalOpenSSL
            elif export_format == 'pkcs8':
                export_format = cryptography.hazmat.primitives.serialization.PrivateFormat.PKCS8
            elif export_format == 'raw':
                export_format = cryptography.hazmat.primitives.serialization.PrivateFormat.Raw
                export_encoding = cryptography.hazmat.primitives.serialization.Encoding.Raw
        except AttributeError:
            self.module.fail_json(msg='Cryptography backend does not support the selected output format "{0}"'.format(self.format))

        # Select key encryption
        encryption_algorithm = cryptography.hazmat.primitives.serialization.NoEncryption()
        if self.cipher and self.passphrase:
            if self.cipher == 'auto':
                encryption_algorithm = cryptography.hazmat.primitives.serialization.BestAvailableEncryption(to_bytes(self.passphrase))
            else:
                self.module.fail_json(msg='Cryptography backend can only use "auto" for cipher option.')

        # Serialize key
        try:
            return self.private_key.private_bytes(
                encoding=export_encoding,
                format=export_format,
                encryption_algorithm=encryption_algorithm
            )
        except ValueError as dummy:
            self.module.fail_json(
                msg='Cryptography backend cannot serialize the private key in the required format "{0}"'.format(self.format)
            )
        except Exception as dummy:
            self.module.fail_json(
                msg='Error while serializing the private key in the required format "{0}"'.format(self.format),
                exception=traceback.format_exc()
            )

    def _load_privatekey(self):
        data = self.existing_private_key_bytes
        try:
            # Interpret bytes depending on format.
            format = identify_private_key_format(data)
            if format == 'raw':
                if len(data) == 56 and CRYPTOGRAPHY_HAS_X448:
                    return cryptography.hazmat.primitives.asymmetric.x448.X448PrivateKey.from_private_bytes(data)
                if len(data) == 57 and CRYPTOGRAPHY_HAS_ED448:
                    return cryptography.hazmat.primitives.asymmetric.ed448.Ed448PrivateKey.from_private_bytes(data)
                if len(data) == 32:
                    if CRYPTOGRAPHY_HAS_X25519 and (self.type == 'X25519' or not CRYPTOGRAPHY_HAS_ED25519):
                        return cryptography.hazmat.primitives.asymmetric.x25519.X25519PrivateKey.from_private_bytes(data)
                    if CRYPTOGRAPHY_HAS_ED25519 and (self.type == 'Ed25519' or not CRYPTOGRAPHY_HAS_X25519):
                        return cryptography.hazmat.primitives.asymmetric.ed25519.Ed25519PrivateKey.from_private_bytes(data)
                    if CRYPTOGRAPHY_HAS_X25519 and CRYPTOGRAPHY_HAS_ED25519:
                        try:
                            return cryptography.hazmat.primitives.asymmetric.x25519.X25519PrivateKey.from_private_bytes(data)
                        except Exception:
                            return cryptography.hazmat.primitives.asymmetric.ed25519.Ed25519PrivateKey.from_private_bytes(data)
                raise PrivateKeyError('Cannot load raw key')
            else:
                return cryptography.hazmat.primitives.serialization.load_pem_private_key(
                    data,
                    None if self.passphrase is None else to_bytes(self.passphrase),
                    backend=self.cryptography_backend
                )
        except Exception as e:
            raise PrivateKeyError(e)

    def _ensure_existing_private_key_loaded(self):
        if self.existing_private_key is None and self.has_existing():
            self.existing_private_key = self._load_privatekey()

    def _check_passphrase(self):
        try:
            format = identify_private_key_format(self.existing_private_key_bytes)
            if format == 'raw':
                # Raw keys cannot be encrypted. To avoid incompatibilities, we try to
                # actually load the key (and return False when this fails).
                self._load_privatekey()
                # Loading the key succeeded. Only return True when no passphrase was
                # provided.
                return self.passphrase is None
            else:
                return cryptography.hazmat.primitives.serialization.load_pem_private_key(
                    self.existing_private_key_bytes,
                    None if self.passphrase is None else to_bytes(self.passphrase),
                    backend=self.cryptography_backend
                )
        except Exception as dummy:
            return False

    def _check_size_and_type(self):
        if isinstance(self.existing_private_key, cryptography.hazmat.primitives.asymmetric.rsa.RSAPrivateKey):
            return self.type == 'RSA' and self.size == self.existing_private_key.key_size
        if isinstance(self.existing_private_key, cryptography.hazmat.primitives.asymmetric.dsa.DSAPrivateKey):
            return self.type == 'DSA' and self.size == self.existing_private_key.key_size
        if CRYPTOGRAPHY_HAS_X25519 and isinstance(self.existing_private_key, cryptography.hazmat.primitives.asymmetric.x25519.X25519PrivateKey):
            return self.type == 'X25519'
        if CRYPTOGRAPHY_HAS_X448 and isinstance(self.existing_private_key, cryptography.hazmat.primitives.asymmetric.x448.X448PrivateKey):
            return self.type == 'X448'
        if CRYPTOGRAPHY_HAS_ED25519 and isinstance(self.existing_private_key, cryptography.hazmat.primitives.asymmetric.ed25519.Ed25519PrivateKey):
            return self.type == 'Ed25519'
        if CRYPTOGRAPHY_HAS_ED448 and isinstance(self.existing_private_key, cryptography.hazmat.primitives.asymmetric.ed448.Ed448PrivateKey):
            return self.type == 'Ed448'
        if isinstance(self.existing_private_key, cryptography.hazmat.primitives.asymmetric.ec.EllipticCurvePrivateKey):
            if self.type != 'ECC':
                return False
            if self.curve not in self.curves:
                return False
            return self.curves[self.curve]['verify'](self.existing_private_key)

        return False

    def _check_format(self):
        if self.format == 'auto_ignore':
            return True
        try:
            format = identify_private_key_format(self.existing_private_key_bytes)
            return format == self._get_wanted_format()
        except Exception as dummy:
            return False


def select_backend(module, backend):
    if backend == 'auto':
        # Detection what is possible
        can_use_cryptography = CRYPTOGRAPHY_FOUND and CRYPTOGRAPHY_VERSION >= LooseVersion(MINIMAL_CRYPTOGRAPHY_VERSION)

        # Decision
        if can_use_cryptography:
            backend = 'cryptography'

        # Success?
        if backend == 'auto':
            module.fail_json(msg=("Cannot detect the required Python library "
                                  "cryptography (>= {0})").format(MINIMAL_CRYPTOGRAPHY_VERSION))
    if backend == 'cryptography':
        if not CRYPTOGRAPHY_FOUND:
            module.fail_json(msg=missing_required_lib('cryptography >= {0}'.format(MINIMAL_CRYPTOGRAPHY_VERSION)),
                             exception=CRYPTOGRAPHY_IMP_ERR)
        return backend, PrivateKeyCryptographyBackend(module)
    else:
        raise Exception('Unsupported value for backend: {0}'.format(backend))


def get_privatekey_argument_spec():
    return ArgumentSpec(
        argument_spec=dict(
            size=dict(type='int', default=4096),
            type=dict(type='str', default='RSA', choices=[
                'DSA', 'ECC', 'Ed25519', 'Ed448', 'RSA', 'X25519', 'X448'
            ]),
            curve=dict(type='str', choices=[
                'secp224r1', 'secp256k1', 'secp256r1', 'secp384r1', 'secp521r1',
                'secp192r1', 'brainpoolP256r1', 'brainpoolP384r1', 'brainpoolP512r1',
                'sect163k1', 'sect163r2', 'sect233k1', 'sect233r1', 'sect283k1',
                'sect283r1', 'sect409k1', 'sect409r1', 'sect571k1', 'sect571r1',
            ]),
            passphrase=dict(type='str', no_log=True),
            cipher=dict(type='str'),
            format=dict(type='str', default='auto_ignore', choices=['pkcs1', 'pkcs8', 'raw', 'auto', 'auto_ignore']),
            format_mismatch=dict(type='str', default='regenerate', choices=['regenerate', 'convert']),
            select_crypto_backend=dict(type='str', choices=['auto', 'cryptography'], default='auto'),
            regenerate=dict(
                type='str',
                default='full_idempotence',
                choices=['never', 'fail', 'partial_idempotence', 'full_idempotence', 'always']
            ),
        ),
        required_together=[
            ['cipher', 'passphrase']
        ],
        required_if=[
            ['type', 'ECC', ['curve']],
        ],
    )



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: nxos_evpn_global
extends_documentation_fragment:
- cisco.nxos.nxos
short_description: Handles the EVPN control plane for VXLAN.
description:
- Handles the EVPN control plane for VXLAN.
version_added: 1.0.0
author: Gabriele Gerbino (@GGabriele)
notes:
- This module is not supported on Nexus 3000 series of switches.
- Unsupported for Cisco MDS
options:
  nv_overlay_evpn:
    description:
    - EVPN control plane.
    required: true
    type: bool
"""

EXAMPLES = """
- cisco.nxos.nxos_evpn_global:
    nv_overlay_evpn: true
"""

RETURN = """
commands:
    description: The set of commands to be sent to the remote device
    returned: always
    type: list
    sample: ['nv overlay evpn']
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    get_capabilities,
    get_config,
    load_config,
)


def main():
    argument_spec = dict(nv_overlay_evpn=dict(required=True, type="bool"))

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    result = {"changed": False}

    warnings = list()
    if warnings:
        result["warnings"] = warnings

    config = get_config(module)
    commands = list()

    info = get_capabilities(module).get("device_info", {})
    os_platform = info.get("network_os_platform", "")

    if "3K" in os_platform:
        module.fail_json(msg="This module is not supported on Nexus 3000 series")

    if module.params["nv_overlay_evpn"] is True:
        if "nv overlay evpn" not in config:
            commands.append("nv overlay evpn")
    elif "nv overlay evpn" in config:
        commands.append("no nv overlay evpn")

    if commands:
        if not module.check_mode:
            load_config(module, commands)
        result["changed"] = True

    result["commands"] = commands

    module.exit_json(**result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Snmp_server parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


def _tmplt_snmp_server_ipv6_comm(config_data):
    command = ""
    if "acl_v6" in config_data["communities"]:
        command = "snmp-server community "
        el = config_data["communities"]
        command += el["name"]
        if el.get("view"):
            command += " view " + el["view"]
        if el.get("ro"):
            command += " ro"
        if el.get("rw"):
            command += " rw"
        command += " ipv6 " + el["acl_v6"]
    return command


def _tmplt_snmp_server_ipv4_comm(config_data):
    command = ""
    if not config_data["communities"].get("acl_v6"):
        command = "snmp-server community "
        el = config_data["communities"]
        command += el["name"]
        if el.get("view"):
            command += " view " + el["view"]
        if el.get("ro"):
            command += " ro"
        if el.get("rw"):
            command += " rw"
        if el.get("acl_v4"):
            command += " " + el["acl_v4"]
    return command


def _tmplt_snmp_server_traps_bgp(config_data):
    command = "snmp-server enable traps bgp"
    el = config_data["traps"]["bgp"]
    if el.get("arista_backward_transition"):
        command += " arista-backward-transition"
    if el.get("arista_established"):
        command += " arista-established"
    if el.get("backward_transition"):
        command += " backward-transition"
    if el.get("established"):
        command += " established"
    return command


def _tmplt_snmp_server_traps_bridge(config_data):
    command = "snmp-server enable traps bridge"
    el = config_data["traps"]["bridge"]
    if el.get("arista_mac_age"):
        command += " arista-mac-age"
    if el.get("arista_mac_learn"):
        command += " arista-mac-learn"
    if el.get("arista_mac_move"):
        command += " arista-mac-move"
    return command


def _tmplt_snmp_server_traps_capacity(config_data):
    command = "snmp-server enable traps capacity"
    el = config_data["traps"]["capacity"]
    if el.get("arista_hardware_utilization_alert"):
        command += " arista-hardware-utilization-alert"
    return command


def _tmplt_snmp_server_traps_entity(config_data):
    command = "snmp-server enable traps entity"
    el = config_data["traps"]["entity"]
    if el.get("arista_ent_sensor_alarm"):
        command += " arista-ent-sensor-alarm"
    if el.get("ent_config_change"):
        command += " ent-config-change"
    if el.get("ent_state_oper"):
        command += " ent-state-oper"
    if el.get("ent_state_oper_disabled"):
        command += " ent-state-oper-disabled"
    if el.get("ent_state_oper_enabled"):
        command += " ent-state-oper-enabled"
    return command


def _tmplt_snmp_server_traps_external_alarm(config_data):
    command = "snmp-server enable traps external-alarm"
    el = config_data["traps"]["external_alarm"]
    if el.get("arista_external_alarm_asserted_notif"):
        command += " arista-external-alarm-asserted-notif"
    if el.get("arista_external_alarm_deasserted_notif"):
        command += " arista-external-alarm-deasserted-notif"
    return command


def _tmplt_snmp_server_traps_isis(config_data):
    command = "snmp-server enable traps isis"
    el = config_data["traps"]["isis"]
    if el.get("adjacency_change"):
        command += " adjacency-change"
    if el.get("area_mismatch"):
        command += " area-mismatch"
    if el.get("attempt_to_exceed_max_sequence"):
        command += " attempt-to-exceed-max-sequence"
    if el.get("authentication_type_failure"):
        command += " authentication-type-failure"
    if el.get("database_overload"):
        command += " database-overload"
    if el.get("own_lsp_purge"):
        command += " own-lsp-purge"
    if el.get("rejected_adjacency"):
        command += " rejected-adjacency"
    if el.get("sequence_number_skip"):
        command += " sequence-number-skip"

    return command


def _tmplt_snmp_server_traps_lldp(config_data):
    command = "snmp-server enable traps lldp"
    el = config_data["traps"]["lldp"]
    if el.get("rem_tables_change"):
        command += " rem-tables-change"
    return command


def _tmplt_snmp_server_traps_mpls_ldp(config_data):
    command = "snmp-server enable traps mpls-ldp"
    el = config_data["traps"]["mpls_ldp"]
    if el.get("mpls_ldp_session_down"):
        command += " mpls-ldp-session-down"
    if el.get("mpls_ldp_session_up"):
        command += " mpls-ldp-session-up"
    return command


def _tmplt_snmp_server_traps_msdp(config_data):
    command = "snmp-server enable traps msdp"
    el = config_data["traps"]["msdp"]
    if el.get("backward_transition"):
        command += " backward-transition"
    if el.get("established"):
        command += " established"
    return command


def _tmplt_snmp_server_traps_ospf(config_data):
    command = "snmp-server enable traps ospf"
    el = config_data["traps"]["ospf"]
    if el.get("if_auth_failure"):
        command += " if-auth-failure"
    if el.get("if_config_error"):
        command += " if-config-error"
    if el.get("if_state_change"):
        command += " if-state-change"
    if el.get("nbr_state_change"):
        command += " nbr-state-change"
    return command


def _tmplt_snmp_server_traps_ospfv3(config_data):
    command = "snmp-server enable traps ospfv3"
    el = config_data["traps"]["ospfv3"]
    if el.get("if_config_error"):
        command += " if-config-error"
    if el.get("if_rx_bad_packet"):
        command += " if-rx-bad-packet"
    if el.get("if_state_change"):
        command += " if-state-change"
    if el.get("nbr_state_change"):
        command += " nbr-state-change"
    if el.get("nbr_restart_helper_status_change"):
        command += " nbr-restart-helper-status-change"
    if el.get("nssa_translator_status_change"):
        command += " nssa-translator-status-change"
    if el.get("restart_status_change"):
        command += " restart-status-change"
    return command


def _tmplt_snmp_server_traps_pim(config_data):
    command = "snmp-server enable traps pim"
    el = config_data["traps"]["pim"]
    if el.get("neighbor_loss"):
        command += " neighbor-loss"
    return command


def _tmplt_snmp_server_traps_snmp(config_data):
    command = "snmp-server enable traps snmp"
    el = config_data["traps"]["snmp"]
    if el.get("authentication"):
        command += " authentication"
    if el.get("link_down"):
        command += " link-down"
    if el.get("link_up"):
        command += " link-up"
    return command


def _tmplt_snmp_server_traps_snmpConfigManEvent(config_data):
    command = "snmp-server enable traps snmpConfigManEvent"
    el = config_data["traps"]["snmpConfigManEvent"]
    if el.get("arista_config_man_event"):
        command += " arista-config-man-event"
    return command


def _tmplt_snmp_server_traps_switchover(config_data):
    command = "snmp-server enable traps switchover"
    el = config_data["traps"]["switchover"]
    if el.get("arista_redundancy_switch_over_notif"):
        command += " arista-redundancy-switch-over-notif"
    return command


def _tmplt_snmp_server_traps_test(config_data):
    command = "snmp-server enable traps test"
    el = config_data["traps"]["test"]
    if el.get("arista_test_notification"):
        command += " arista-test-notification"
    return command


def _tmplt_snmp_server_traps_vrrp(config_data):
    command = "snmp-server enable traps vrrp"
    el = config_data["traps"]["vrrp"]
    if el.get("trap_new_master"):
        command += " trap-new-master"
    return command


def _tmplt_snmp_server_engineid(config_data):
    command = []
    cmd = "snmp-server engineID"
    el = config_data["engineid"]
    if el.get("local"):
        c = cmd + " local " + el["local"]
        command.append(c)
    if el.get("remote"):
        c = cmd + " remote " + el["remote"]["host"]
        if el["remote"].get("udp_port"):
            c += " udp-port " + str(el["remote"]["udp_port"])
        if el["remote"].get("id"):
            c += " " + el["remote"]["id"]
            command.append(c)
    return command


def _tmplt_snmp_server_extension(config_data):
    command = "snmp-server extension "
    command += config_data["extension"]["root_oid"]
    command += " " + config_data["extension"]["script_location"]
    if config_data["extension"].get("oneshot"):
        command += " one-shot"
    return command


def _tmplt_snmp_server_groups(config_data):
    command = "snmp-server group " + config_data["groups"]["group"]
    el = config_data["groups"]
    command += " " + el["version"]
    if el.get("auth_privacy"):
        command += " " + el["auth_privacy"]
    for param in ["context", "read", "write", "notify"]:
        if el.get(param):
            command += " " + param + " " + el[param]
    return command


def _tmplt_snmp_server_hosts(config_data):
    el = list(config_data["hosts"].values())[0]
    command = "snmp-server host " + el["host"]
    if el.get("vrf"):
        command += " vrf" + el["vrf"]
    if el.get("informs"):
        command += " informs"
    if el.get("traps"):
        command += " traps"
    if el.get("version"):
        command += " version " + el["version"]
    if el.get("user"):
        command += " " + el["user"]
    if el.get("udp_port"):
        command += " udp-port " + str(el["udp_port"])
    return command


def _tmplt_snmp_server_acls(config_data):
    command = "snmp-server " + config_data["acls"]["afi"] + " access-list "
    el = config_data["acls"]
    command += el["acl"]
    if el.get("vrf"):
        command += " vrf " + el["vrf"]
    return command


def _tmplt_snmp_server_vrfs(config_data):
    command = "snmp-server vrf " + config_data["vrfs"]["vrf"]
    el = config_data["vrfs"]
    if el.get("local_interface"):
        command += " local-interface " + el["local_interface"]
    return command


def _tmplt_snmp_server_users_auth(config_data):
    el = config_data["users"]
    command = "snmp-server user " + el["user"] + " " + el["group"]
    if el.get("remote"):
        command += " remote " + el["remote"]
    if el.get("udp_port"):
        command += " udp-port " + str(el["udp_port"])
    command += " " + el["version"]
    if el.get("auth"):
        command += " " + el["auth"]["algorithm"] + " " + el["auth"]["auth_passphrase"]
        if el["auth"].get("encryption"):
            command += " priv " + el["auth"]["encryption"] + " " + el["auth"]["priv_passphrase"]
    return command


def _tmplt_snmp_server_users_localized(config_data):
    el = config_data["users"]
    command = "snmp-server user " + el["user"] + " " + el["group"]
    if el.get("remote"):
        command += " remote " + el["remote"]
    if el.get("udp_port"):
        command += " udp-port " + str(el["udp_port"])
    command += " " + el["version"]
    if el.get("localized"):
        command += " localized " + el["localized"]["engineid"]
        el = el["localized"]
        command += " " + el["algorithm"] + " " + el["auth_passphrase"]
        if el.get("encryption"):
            command += " priv " + el["encryption"] + " " + el["priv_passphrase"]
    return command


class Snmp_serverTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Snmp_serverTemplate, self).__init__(
            lines=lines,
            tmplt=self,
            module=module,
        )

    # fmt: off
    PARSERS = [
        {
            "name": "chassis_id",
            "getval": re.compile(
                r"""
                \s*snmp-server\schassis-id
                \s*(?P<id>\S+)*
                $""",
                re.VERBOSE,
            ),
            "setval": 'snmp-server chassis-id {{ chassis_id }}',
            "result": {
                "chassis_id": "{{ id }}",
            },
        },
        {
            "name": "communities_ipv6_acl",
            "getval": re.compile(
                r"""
                \s*snmp-server\scommunity
                \s+(?P<comm>\S+)
                \s*(?P<view>view\s\S+)*
                \s*(?P<access>ro|rw)*
                \s*(?P<acl>ipv6\s\S+)
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_ipv6_comm,
            "compval": "communities",
            "result": {
                "communities": {
                    "{{ comm }}": {
                        "name": "{{ comm }}",
                        "acl_v6": "{{ acl.split(" ")[1] }}",
                        "view": "{{ view.split(" ")[1] if view is defined }}",
                        "ro": '{{ True if access == "ro" }}',
                        "rw": '{{ True if access == "rw" }}',
                    },
                },
            },
        },
        {
            "name": "communities_ipv4_acl",
            "getval": re.compile(
                r"""
                \s*snmp-server\scommunity
                \s+(?P<comm>\S+)
                \s*(?P<view>view\s\S+)*
                \s*(?P<access>ro|rw)*
                \s*(?P<acl>\S+)
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_ipv4_comm,
            "compval": "communities",
            "result": {
                "communities": {
                    "{{ comm }}": {
                        "name": "{{ comm }}",
                        "acl_v4": '{{ acl if acl != "ipv6" }}',
                        "view": "{{ view.split(" ")[1] if view is defined }}",
                        "ro": '{{ True if access == "ro" }}',
                        "rw": '{{ True if access == "rw" }}',
                    },
                },
            },
        },
        {
            "name": "contact",
            "getval": re.compile(
                r"""
                \s*snmp-server\scontact
                \s+(?P<name>.+)
                *$""",
                re.VERBOSE,
            ),
            "setval": 'snmp-server contact {{ contact }}',
            "compval": "contact",
            "result": {
                "contact": "{{ name }}",
            },
        },
        {
            "name": "traps.bgp",
            "getval": re.compile(
                r"""
                \s*snmp-server\senable\straps\sbgp
                \s*(?P<trap1>arista-backward-transition)*
                \s*(?P<trap2>arista-established)*
                \s*(?P<trap3>backward-transition)*
                \s*(?P<trap4>established)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_traps_bgp,
            "result": {
                "traps": {
                    "bgp": {
                        "arista_backward_transition": "{{ True if trap1 is defined }}",
                        "arista_established": "{{ True if trap2 is defined }}",
                        "backward_transition": "{{ True if trap3 is defined }}",
                        "established": "{{ True if trap4 is defined }}",
                        "enabled": "{{ True if trap1 is undefined and trap2 is undefined and trap3 is undefined and trap4 is undefined }}",
                    },
                },
            },
        },
        {
            "name": "traps.bridge",
            "getval": re.compile(
                r"""
                \s*snmp-server\senable\straps\sbridge
                \s*(?P<trap1>arista-mac-age)*
                \s*(?P<trap2>arista-mac-learn)*
                \s*(?P<trap3>arista-mac-move)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_traps_bridge,
            "result": {
                "traps": {
                    "bridge": {
                        "enabled": "{{ True if trap1 is undefined and trap2 is undefined and trap3 is undefined }}",
                        "arista_mac_age": "{{ True if trap1 is defined }}",
                        "arista_mac_learn": "{{ True if trap2 is defined }}",
                        "arista_mac_move": "{{ True if trap3 is defined }}",
                    },
                },
            },
        },

        {
            "name": "traps.capacity",
            "getval": re.compile(
                r"""
                \s*snmp-server\senable\straps\scapacity
                \s*(?P<trap1>arista-hardware-utilization-alert)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_traps_capacity,
            "result": {
                "traps": {
                    "capacity": {
                        "arista_hardware_utilization_alert": "{{ True if trap1 is defined }}",
                        "enabled": "{{ True if trap1 is undefined }}",
                    },
                },
            },
        },
        {
            "name": "traps.entity",
            "getval": re.compile(
                r"""
                \s*snmp-server\senable\straps\sentity
                \s*(?P<trap1>arista-ent-sensor-alarm)*
                \s*(?P<trap2>ent-config-change)*
                \s*(?P<trap3>ent-state-oper)*
                \s*(?P<trap4>ent-state-oper-disabled)*
                \s*(?P<trap5>ent-state-oper-enabled)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_traps_entity,
            "result": {
                "traps": {
                    "entity": {
                        "arista_ent_sensor_alarm": "{{ True if trap1 is defined }}",
                        "ent_config_change": "{{ True if trap2 is defined }}",
                        "ent_state_oper": "{{ True if trap3 is defined }}",
                        "ent_state_oper_disabled": "{{ True if trap4 is defined }}",
                        "ent_state_oper_enabled": "{{ True if trap4 is defined }}",
                        "enabled": "{{ True if trap1 is undefined and trap2 is undefined and trap3 is undefined\
                             and trap4 is undefined and trap5 is undefined }}",
                    },
                },
            },
        },
        {
            "name": "traps.external_alarm",
            "getval": re.compile(
                r"""
                \s*snmp-server\senable\straps\sexternal-alarm
                \s*(?P<trap1>arista-external-alarm-asserted-notif)*
                \s*(?P<trap2>arista-external-alarm-deasserted-notif)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_traps_external_alarm,
            "result": {
                "traps": {
                    "external_alarm": {
                        "arista_external_alarm_asserted_notif": "{{ True if trap1 is defined }}",
                        "arista_external_alarm_deasserted_notif": "{{ True if trap2 is defined }}",
                        "enabled": "{{ True if trap1 is undefined and trap2 is undefined }}",
                    },
                },
            },
        },
        {
            "name": "traps.isis",
            "getval": re.compile(
                r"""
                \s*snmp-server\senable\straps\sisis
                \s*(?P<trap1>adjacency-change)*
                \s*(?P<trap2>area-mismatch)*
                \s*(?P<trap3>attempt-to-exceed-max-sequence)*
                \s*(?P<trap4>authentication-type-failure)*
                \s*(?P<trap5>database-overload)*
                \s*(?P<trap6>own-lsp-purge)*
                \s*(?P<trap7>rejected-adjacency)*
                \s*(?P<trap8>equence-number-skip)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_traps_isis,
            "result": {
                "traps": {
                    "isis": {
                        "adjacency_change": "{{ True if trap1 is defined }}",
                        "area_mismatch": "{{ True if trap2 is defined }}",
                        "attempt_to_exceed_max_sequence": "{{ True if trap3 is defined }}",
                        "authentication_type_failure": "{{ True if trap4 is defined }}",
                        "database_overload": "{{ True if trap4 is defined }}",
                        "own_lsp_purge": "{{ True if trap4 is defined }}",
                        "rejected_adjacency": "{{ True if trap4 is defined }}",
                        "sequence_number_skip": "{{ True if trap4 is defined }}",
                        "enabled": "{{ True if trap1 is undefined and trap2 is undefined and trap3 is undefined and trap4 is undefined\
                                    and trap5 is undefined and trap6 is undefined and trap7 is undefined and trap8 is undefined }}",

                    },
                },
            },
        },
        {
            "name": "traps.lldp",
            "getval": re.compile(
                r"""
                \s*snmp-server\senable\straps\slldp
                \s*(?P<trap1>rem-tables-change)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_traps_lldp,
            "result": {
                "traps": {
                    "lldp": {
                        "rem_tables_change": "{{ True if trap1 is defined }}",
                        "enabled": "{{ True if trap1 is undefined }}",
                    },
                },
            },
        },
        {
            "name": "traps.mpls_ldp",
            "getval": re.compile(
                r"""
                \s*snmp-server\senable\straps\smpls-ldp
                \s*(?P<trap1>mpls-ldp-session-down)*
                \s*(?P<trap2>mpls-ldp-session-up)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_traps_mpls_ldp,
            "result": {
                "traps": {
                    "mpls_ldp": {
                        "mpls_ldp_session_down": "{{ True if trap1 is defined }}",
                        "mpls_ldp_session_up": "{{ True if trap2 is defined }}",
                        "enabled": "{{ True if trap1 is undefined and trap2 is undefined }}",
                    },
                },
            },
        },
        {
            "name": "traps.msdp",
            "getval": re.compile(
                r"""
                \s*snmp-server\senable\straps\smsdp
                \s*(?P<trap1>backward-transition)*
                \s*(?P<trap2>established)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_traps_msdp,
            "result": {
                "traps": {
                    "msdp": {
                        "backward_transition": "{{ True if trap1 is defined }}",
                        "established": "{{ True if trap2 is defined }}",
                        "enabled": "{{ True if trap1 is undefined and trap2 is undefined }}",
                    },
                },
            },
        },
        {
            "name": "traps.ospf",
            "getval": re.compile(
                r"""
                \s*snmp-server\senable\straps\sospf
                \s*(?P<trap1>if-auth-failure)*
                \s*(?P<trap2>if-config-error)*
                \s*(?P<trap3>if-state-change)*
                \s*(?P<trap4>nbr-state-change)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_traps_ospf,
            "result": {
                "traps": {
                    "ospf": {
                        "if_config_error": "{{ True if trap2 is defined }}",
                        "if_auth_failure": "{{ True if trap1 is defined }}",
                        "if_state_change": "{{ True if trap3 is defined }}",
                        "nbr_state_change": "{{ True if trap4 is defined }}",
                        "enabled": "{{ True if trap1 is undefined and trap2 is undefined and trap3 is undefined and trap4 is undefined }}",
                    },
                },
            },
        },
        {
            "name": "traps.ospfv3",
            "getval": re.compile(
                r"""
                \s*snmp-server\senable\straps\sospfv3
                \s*(?P<trap1>if-config-error)*
                \s*(?P<trap2>if-rx-bad-packet)*
                \s*(?P<trap3>if-state-change)*
                \s*(?P<trap4>nbr-restart-helper-status-change)*
                \s*(?P<trap5>nbr-state-change)*
                \s*(?P<trap6>nssa-translator-status-change)*
                \s*(?P<trap7>restart-status-change)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_traps_ospfv3,
            "result": {
                "traps": {
                    "ospfv3": {
                        "if_config_error": "{{ True if trap1 is defined }}",
                        "if_rx_bad_packet": "{{ True if trap2 is defined }}",
                        "if_state_change": "{{ True if trap3 is defined }}",
                        "nbr_state_change": "{{ True if trap5 is defined }}",
                        "nbr_restart_helper_status_change": "{{ True if trap4 is defined }}",
                        "nssa_translator_status_change": "{{ True if trap6 is defined }}",
                        "restart_status_change": "{{ True if trap7 is defined }}",
                        "enabled": "{{ True if trap1 is undefined and trap2 is undefined and trap3 is undefined and trap4 is undefined\
                         and trap5 is undefined and trap6 is undefined and trap7 is undefined }}",
                    },
                },
            },
        },
        {
            "name": "traps.pim",
            "getval": re.compile(
                r"""
                \s*snmp-server\senable\straps\spim
                \s*(?P<trap1>neighbor-loss)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_traps_pim,
            "result": {
                "traps": {
                    "pim": {
                        "neighbor_loss": "{{ True if trap1 is defined }}",
                        "enabled": "{{ True if trap1 is undefined }}",
                    },
                },
            },
        },
        {
            "name": "traps.snmp",
            "getval": re.compile(
                r"""
                \s*snmp-server\senable\straps\ssnmp
                \s*(?P<trap1>authentication)*
                \s*(?P<trap2>link-down)*
                \s*(?P<trap3>link-up)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_traps_snmp,
            "result": {
                "traps": {
                    "snmp": {
                        "authentication": "{{ True if trap1 is defined }}",
                        "link_down": "{{ True if trap2 is defined }}",
                        "link_up": "{{ True if trap3 is defined }}",
                        "enabled": "{{ True if trap1 is undefined and trap2 is undefined and trap3 is undefined }}",
                    },
                },
            },
        },
        {
            "name": "traps.snmpConfigManEvent",
            "getval": re.compile(
                r"""
                \s*snmp-server\senable\straps\ssnmpConfigManEvent
                \s*(?P<trap1>arista-config-man-event)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_traps_snmpConfigManEvent,
            "result": {
                "traps": {
                    "snmpConfigManEvent": {
                        "arista_config_man_event": "{{ True if trap1 is defined }}",
                        "enabled": "{{ True if trap1 is undefined }}",
                    },
                },
            },
        },
        {
            "name": "traps.switchover",
            "getval": re.compile(
                r"""
                \s*snmp-server\senable\straps\sswitchover
                \s*(?P<trap1>arista-redundancy-switch-over-notif)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_traps_switchover,
            "result": {
                "traps": {
                    "switchover": {
                        "arista_redundancy_switch_over_notif": "{{ True if trap1 is defined }}",
                        "enabled": "{{ True if trap1 is undefined }}",
                    },
                },
            },
        },
        {
            "name": "traps.test",
            "getval": re.compile(
                r"""
                \s*snmp-server\senable\straps\stest
                \s*(?P<trap1>arista-test-notification)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_traps_test,
            "result": {
                "traps": {
                    "test": {
                        "arista_test_notification": "{{ True if trap1 is defined }}",
                        "enabled": "{{ True if trap1 is undefined }}",
                    },
                },
            },
        },
        {
            "name": "traps.vrrp",
            "getval": re.compile(
                r"""
                \s*snmp-server\senable\straps\svrrp
                \s*(?P<trap1>trap-new-master)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_traps_vrrp,
            "result": {
                "traps": {
                    "vrrp": {
                        "trap_new_master": "{{ True if trap1 is defined }}",
                        "enabled": "{{ True if trap1 is undefined }}",
                    },
                },
            },
        },
        {
            "name": "engineid",
            "getval": re.compile(
                r"""
                \s*snmp-server\sengineID
                \s*(?P<local>local \S+)*
                \s*(?P<remote>remote \S+)*
                \s*(?P<udp>udp-port\s\d+)*
                \s*(?P<id>\S+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_engineid,
            "result": {
                "engineid": {
                    "local": "{{ local.split(" ")[1] if local is defined }}",
                    "remote": {
                        "host": "{{ remote.split(" ")[1] if remote is defined }}",
                        "id": "{{ id }}",
                        "udp_port": "{{ udp.split(" ")[1] if udp is defined }}",
                    },
                },
            },
        },
        {
            "name": "extension",
            "getval": re.compile(
                r"""
                \s*snmp-server\sextension
                \s*(?P<oid>\.\S+|str)*
                \s*(?P<script>\S+)*
                \s*(?P<oneshot>one-shot)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_extension,
            "remval": "snmp-server extension {{ extension.root_oid.lstrip('0') }} {{ extension.script_location }}",
            "result": {
                "extension": {
                    "root_oid": "{{ oid }}",
                    "script_location": "{{ script }}",
                    "oneshot": "{{ True if oneshot is defined }}",
                },
            },
        },
        {
            "name": "groups",
            "getval": re.compile(
                r"""
                \s*snmp-server\sgroup
                \s*(?P<name>\S+)*
                \s*(?P<version>v1|v2c|v3\sauth|v3\snoauth|v3\spriv)*
                \s*(?P<context>context\s\S+)*
                \s*(?P<read>read\s\S+)*
                \s*(?P<write>write\s\S+)*
                \s*(?P<notify>notify\s\S+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_groups,
            "result": {
                "groups": {
                    "{{ name }}": {
                        "group": "{{ name }}",
                        "version": "{{ version.split(" ")[0] }}",
                        "auth_privacy": '{{ version.split(" ")[1] if "v3" in version }}',
                        "context": "{{ context.split(" ")[1] if context is defined }}",
                        "notify": "{{ notify.split(" ")[1] if notify is defined }}",
                        "read": "{{ read.split(" ")[1] if read is defined }}",
                        "write": "{{ write.split(" ")[1] if write is defined }}",
                    },
                },
            },
        },
        {
            "name": "hosts",
            "getval": re.compile(
                r"""
                \s*snmp-server\shost
                \s(?P<name>\S+)
                \s*(?P<vrf>vrf\s\S+)*
                \s*(?P<msg_inf1>informs)*
                \s*(?P<msg_tr1>traps)*
                \s*(version)*
                \s*(?P<version>1|2c|3\sauth|3\snoauth|3\spriv)*
                \s*(?P<msg_inf2>informs)*
                \s*(?P<msg_tr2>traps)*
                \s*(?P<comm>\S+)*
                \s*(?P<udp>udp-port)*?
                \s*(?P<port>\d+)*?
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_hosts,
            "result": {
                "hosts": {
                    '{{ name, comm|d(""), version|d("2c"), msg_inf1|d() or msg_inf2|d(), msg_tr1|d() or msg_tr2|d(), port|d() }}': {
                        "host": "{{ name }}",
                        "vrf": "{{ vrf.split(" ")[1] if vrf is defined }}",
                        "version": '{{ version }}',
                        "udp_port": "{{ port if udp is defined and port is defined else None }}",
                        "informs": '{{ True if msg_inf1 is defined or msg_inf2 is defined else None }}',
                        "traps": '{{ True if msg_tr1 is defined or msg_tr2 is defined else None }}',
                        "user": "{{ comm }}",
                    },
                },
            },
        },
        {
            "name": "acls",
            "getval": re.compile(
                r"""
                \s*snmp-server\s
                \s+(?P<afi>ipv4|ipv6)
                \s+access-list
                \s+(?P<acl>\S+)
                \s*(?P<vrf>vrf\s\S+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_acls,
            "result": {
                "acls": {
                    "{{ afi }}": {
                        "afi": "{{ afi }}",
                        "acl": "{{ acl }}",
                        "vrf": "{{ vrf.split(" ")[1] if vrf is defined }}",
                    },
                },
            },
        },
        {
            "name": "local_interface",
            "getval": re.compile(
                r"""
                \s*snmp-server\slocal-interface
                \s+(?P<int>.+)
                *$""",
                re.VERBOSE,
            ),
            "setval": "snmp-server local-interface {{ local_interface }}",
            "result": {
                "local_interface": "{{ int }}",
            },
        },
        {
            "name": "location",
            "getval": re.compile(
                r"""
                \s*snmp-server\slocation
                \s+(?P<loc>.+)
                *$""",
                re.VERBOSE,
            ),
            "setval": "snmp-server location {{ location }}",
            "result": {
                "location": "{{ loc }}",
            },
        },
        {
            "name": "notification",
            "getval": re.compile(
                r"""
                \s*snmp-server\snotification\slog\sentry\slimit
                \s+(?P<num>\d+)
                *$""",
                re.VERBOSE,
            ),
            "setval": "snmp-server notification log entry limit {{ notification }}",
            "result": {
                "notification": "{{ num }}",
            },
        },
        {
            "name": "objects.mac",
            "getval": re.compile(
                r"""
                \s*snmp-server\sobjects
                \s+mac-address-tables
                \s+disable
                *$""",
                re.VERBOSE,
            ),
            "setval": "snmp-server objects mac-address-tables disable",
            "compval": "objects",
            "result": {
                "objects": {
                    "mac_address_tables": "{{ True }}",
                },
            },
        },
        {
            "name": "objects.route",
            "getval": re.compile(
                r"""
                \s*snmp-server\sobjects
                \s+route-tables
                \s+disable
                *$""",
                re.VERBOSE,
            ),
            "setval": "snmp-server objects route-tables disable",
            "compval": "objects",
            "result": {
                "objects": {
                    "route_address_tables": "{{ True }}",
                },
            },
        },
        {
            "name": "qos",
            "getval": re.compile(
                r"""
                \s*snmp-server\sqos\sdscp
                \s+(?P<num>\d+)
                *$""",
                re.VERBOSE,
            ),
            "setval": "snmp-server qos dscp {{ qos }}",
            "result": {
                "qos": "{{ num }}",
            },
        },
        {
            "name": "qosmib",
            "getval": re.compile(
                r"""
                \s*snmp-server\sqosmib\scounter-interval
                \s+(?P<num>\d+)
                *$""",
                re.VERBOSE,
            ),
            "setval": "snmp-server qosmib counter-interval {{ qosmib }}",
            "result": {
                "qosmib": "{{ num }}",
            },
        },
        {
            "name": "transmit",
            "getval": re.compile(
                r"""
                \s*snmp-server\stransmit\smax-size
                \s+(?P<num>\d+)
                *$""",
                re.VERBOSE,
            ),
            "setval": "snmp-server transmit max-size {{ transmit }}",
            "result": {
                "transmit": "{{ num }}",
            },
        },
        {
            "name": "transport",
            "getval": re.compile(
                r"""
                \s*snmp-server\stransport\stcp
                *$""",
                re.VERBOSE,
            ),
            "setval": "snmp-server transport tcp",
            "result": {
                "transport": '{{ "tcp" }}',
            },
        },
        {
            "name": "views",
            "getval": re.compile(
                r"""
                \s*snmp-server\sview
                \s+(?P<name>\S+)
                \s+(?P<mib>\S+)
                \s+(?P<action>excluded|included)
                *$""",
                re.VERBOSE,
            ),
            "setval": "snmp-server view {{ views.view }} {{ views.mib }} {{ views.action }}",
            "result": {
                "views": {
                    "{{ name }}": {
                        "view": "{{ name }}",
                        "mib": "{{ mib }}",
                        "action": "{{ actions }}",
                    },
                },
            },
        },
        {
            "name": "vrfs",
            "getval": re.compile(
                r"""
                \s*snmp-server\svrf
                \s+(?P<name>\S+)
                \s*(?P<int>local-interface\s.+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_vrfs,
            "result": {
                "vrfs": {
                    "{{ name }}": {
                        "vrf": "{{ name }}",
                        "local_interface": "{{ int.split(" ")[1] }}",
                    },
                },
            },
        },
        {
            "name": "users.auth",
            "getval": re.compile(
                r"""
                \s*snmp-server\suser
                \s*(?P<name>\S+)*
                \s*(?P<group>\S+)*
                \s*(?P<rem>remote\s\S+)*
                \s*(?P<udp>udp-port\s\d+)*
                \s*(?P<version>v1|v2c|v3)*
                \s*(?P<auth>auth)*
                \s*(?P<algo>\S+)*
                \s*(?P<pass>\S+)*
                \s*(?P<priv>priv)*
                \s*(?P<enc>\S+)*
                \s*(?P<privpass>\S+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_users_auth,
            "result": {
                "users": {
                    "{{ name }}": {
                        "user": "{{ host }}",
                        "group": "{{ group }}",
                        "remote": "{{ rem.split(" ")[1] if rem is defined }}",
                        "version": '{{ version }}',
                        "auth": {
                            "algorithm": "{{ algo }}",
                            "auth_passphrase": "{{ pass }}",
                            "encryption": "{{ enc }}",
                            "priv_passphrase": "{{ privpass }}",
                        },
                        "udp_port": "{{ udp.split(" ")[1] if udp is defined }}",
                    },
                },
            },
        },
        {
            "name": "users.localized",
            "getval": re.compile(
                r"""
                \s*snmp-server\suser
                \s*(?P<name>\S+)*
                \s*(?P<group>\S+)*
                \s*(?P<rem>remote\s\S+)*
                \s*(?P<udp>udp-port\s\d+)*
                \s*(?P<version>v1|v2c|v3)*
                \s*(?P<localized>localized \S+)*
                \s*(?P<algo>\S+)*
                \s*(?P<pass>\S+)*
                \s*(?P<priv>priv)*
                \s*(?P<enc>\S+)*
                \s*(?P<privpass>\S+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_snmp_server_users_localized,
            "result": {
                "users": {
                    "{{ name }}": {
                        "user": "{{ host }}",
                        "group": "{{ group }}",
                        "remote": "{{ rem.split(" ")[1] if rem is defined }}",
                        "version": '{{ version }}',
                        "localized": {
                            "engineid": "{{ localized.split(" ")[1] }}",
                            "algorithm": "{{ algo }}",
                            "auth_passphrase": "{{ pass }}",
                            "encryption": "{{ enc }}",
                            "priv_passphrase": "{{ privpass }}",
                        },
                        "udp_port": "{{ udp.split(" ")[1] if udp is defined }}",
                    },
                },
            },
        },
    ]
    # fmt: on



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: endpoint_certificate
short_description: Resource module for Endpoint Certificate
description:
- Manage operation update of the resource Endpoint Certificate.
- This API allows the client to create an endpoint certificate.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  certTemplateName:
    description: Name of an Internal CA template.
    type: str
  certificateRequest:
    description: Key value map. Must have CN and SAN entries.
    suboptions:
      cn:
        description: Matches the requester's User Name, unless the Requester is an ERS
          Admin. ERS Admins are allowed to create requests for any CN.
        type: str
      san:
        description: Valid MAC Address, delimited by '-'.
        type: str
    type: dict
  dirPath:
    description: Directory absolute path. Defaults to the current working directory.
    type: str
  filename:
    description: The filename used to save the download file.
    type: str
  format:
    description: Allowed values - PKCS12, - PKCS12_CHAIN, - PKCS8, - PKCS8_CHAIN.
    type: str
  password:
    description: Protects the private key. Must have more than 8 characters, less than
      15 characters, at least one upper case letter, at least one lower case letter,
      at least one digit, and can only contain A-Za-z0-9_#.
    type: str
  saveFile:
    description: Enable or disable automatic file creation of raw response.
    type: bool
requirements:
- ciscoisesdk >= 2.1.1
- python >= 3.5
seealso:
- name: Cisco ISE documentation for EndpointCertificate
  description: Complete reference of the EndpointCertificate API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!endpointcert
notes:
  - SDK Method used are
    endpoint_certificate.EndpointCertificate.create_endpoint_certificate,

  - Paths used are
    put /ers/config/endpointcert/certRequest,

"""

EXAMPLES = r"""
- name: Create
  cisco.ise.endpoint_certificate:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    certTemplateName: string
    certificateRequest:
      cn: string
      san: string
    dirPath: /tmp/downloads/
    filename: download_filename.extension
    format: string
    password: string
    saveFile: true

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "data": "filecontent",
      "filename": "filename",
      "dirpath": "download/directory",
      "path": "download/directory/filename"
    }
"""



# -*- coding: utf-8 -*-
# Copyright (C) 2020 Stanislav German-Evtushenko (@giner) <ginermail@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = '''
  name: dict_kv
  short_description: Convert a value to a dictionary with a single key-value pair
  version_added: 1.3.0
  author: Stanislav German-Evtushenko (@giner)
  description:
    - Convert a value to a dictionary with a single key-value pair.
  positional: key
  options:
    _input:
      description: The value for the single key-value pair.
      type: any
      required: true
    key:
      description: The key for the single key-value pair.
      type: any
      required: true
'''

EXAMPLES = '''
- name: Create a one-element dictionary from a value
  ansible.builtin.debug:
    msg: "{{ 'myvalue' | dict_kv('mykey') }}"
    # Produces the dictionary {'mykey': 'myvalue'}
'''

RETURN = '''
  _value:
    description: A dictionary with a single key-value pair.
    type: dictionary
'''


def dict_kv(value, key):
    '''Return a dictionary with a single key-value pair

    Example:

        - hosts: localhost
          gather_facts: false
          vars:
            myvar: myvalue
          tasks:
          - debug:
              msg: "{{ myvar | dict_kv('thatsmyvar') }}"

        produces:

        ok: [localhost] => {
            "msg": {
                "thatsmyvar": "myvalue"
            }
        }

    Example 2:

        - hosts: localhost
          gather_facts: false
          vars:
            common_config:
              type: host
              database: all
            myservers:
            - server1
            - server2
          tasks:
          - debug:
              msg: "{{ myservers | map('dict_kv', 'server') | map('combine', common_config) }}"

        produces:

        ok: [localhost] => {
            "msg": [
                {
                    "database": "all",
                    "server": "server1",
                    "type": "host"
                },
                {
                    "database": "all",
                    "server": "server2",
                    "type": "host"
                }
            ]
        }
    '''
    return {key: value}


class FilterModule(object):
    ''' Query filter '''

    def filters(self):
        return {
            'dict_kv': dict_kv
        }



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Aaron Smith <ajsmith10381@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: iam_password_policy
version_added: 1.0.0
version_added_collection: community.aws
short_description: Update an IAM Password Policy
description:
  - Module updates an IAM Password Policy on a given AWS account
author:
  - "Aaron Smith (@slapula)"
options:
  state:
    description:
      - Specifies the overall state of the password policy.
    required: true
    choices: ['present', 'absent']
    type: str
  min_pw_length:
    description:
      - Minimum password length.
    default: 6
    aliases: [minimum_password_length]
    type: int
  require_symbols:
    description:
      - Require symbols in password.
    default: false
    type: bool
  require_numbers:
    description:
      - Require numbers in password.
    default: false
    type: bool
  require_uppercase:
    description:
      - Require uppercase letters in password.
    default: false
    type: bool
  require_lowercase:
    description:
      - Require lowercase letters in password.
    default: false
    type: bool
  allow_pw_change:
    description:
      - Allow users to change their password.
    default: false
    type: bool
    aliases: [allow_password_change]
  pw_max_age:
    description:
      - Maximum age for a password in days. When this option is 0 then passwords
        do not expire automatically.
    default: 0
    aliases: [password_max_age]
    type: int
  pw_reuse_prevent:
    description:
      - Prevent re-use of passwords.
    default: 0
    aliases: [password_reuse_prevent, prevent_reuse]
    type: int
  pw_expire:
    description:
      - Prevents users from change an expired password.
    default: false
    type: bool
    aliases: [password_expire, expire]
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Password policy for AWS account
  amazon.aws.iam_password_policy:
    state: present
    min_pw_length: 8
    require_symbols: false
    require_numbers: true
    require_uppercase: true
    require_lowercase: true
    allow_pw_change: true
    pw_max_age: 60
    pw_reuse_prevent: 5
    pw_expire: false
"""

RETURN = r""" # """

try:
    import botocore
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule


class IAMConnection(object):
    def __init__(self, module):
        try:
            self.connection = module.resource("iam")
            self.module = module
        except Exception as e:
            module.fail_json(msg=f"Failed to connect to AWS: {str(e)}")

    def policy_to_dict(self, policy):
        policy_attributes = [
            "allow_users_to_change_password",
            "expire_passwords",
            "hard_expiry",
            "max_password_age",
            "minimum_password_length",
            "password_reuse_prevention",
            "require_lowercase_characters",
            "require_numbers",
            "require_symbols",
            "require_uppercase_characters",
        ]
        ret = {}
        for attr in policy_attributes:
            ret[attr] = getattr(policy, attr)
        return ret

    def update_password_policy(self, module, policy):
        min_pw_length = module.params.get("min_pw_length")
        require_symbols = module.params.get("require_symbols")
        require_numbers = module.params.get("require_numbers")
        require_uppercase = module.params.get("require_uppercase")
        require_lowercase = module.params.get("require_lowercase")
        allow_pw_change = module.params.get("allow_pw_change")
        pw_max_age = module.params.get("pw_max_age")
        pw_reuse_prevent = module.params.get("pw_reuse_prevent")
        pw_expire = module.params.get("pw_expire")

        update_parameters = dict(
            MinimumPasswordLength=min_pw_length,
            RequireSymbols=require_symbols,
            RequireNumbers=require_numbers,
            RequireUppercaseCharacters=require_uppercase,
            RequireLowercaseCharacters=require_lowercase,
            AllowUsersToChangePassword=allow_pw_change,
            HardExpiry=pw_expire,
        )
        if pw_reuse_prevent:
            update_parameters.update(PasswordReusePrevention=pw_reuse_prevent)
        if pw_max_age:
            update_parameters.update(MaxPasswordAge=pw_max_age)

        try:
            original_policy = self.policy_to_dict(policy)
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            original_policy = {}

        try:
            results = policy.update(**update_parameters)
            policy.reload()
            updated_policy = self.policy_to_dict(policy)
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            self.module.fail_json_aws(e, msg="Couldn't update IAM Password Policy")

        changed = original_policy != updated_policy
        return (changed, updated_policy, camel_dict_to_snake_dict(results))

    def delete_password_policy(self, policy):
        try:
            results = policy.delete()
        except is_boto3_error_code("NoSuchEntity"):
            self.module.exit_json(changed=False, task_status={"IAM": "Couldn't find IAM Password Policy"})
        except (
            botocore.exceptions.ClientError,
            botocore.exceptions.BotoCoreError,
        ) as e:  # pylint: disable=duplicate-except
            self.module.fail_json_aws(e, msg="Couldn't delete IAM Password Policy")
        return camel_dict_to_snake_dict(results)


def main():
    module = AnsibleAWSModule(
        argument_spec={
            "state": dict(choices=["present", "absent"], required=True),
            "min_pw_length": dict(type="int", aliases=["minimum_password_length"], default=6),
            "require_symbols": dict(type="bool", default=False),
            "require_numbers": dict(type="bool", default=False),
            "require_uppercase": dict(type="bool", default=False),
            "require_lowercase": dict(type="bool", default=False),
            "allow_pw_change": dict(type="bool", aliases=["allow_password_change"], default=False),
            "pw_max_age": dict(type="int", aliases=["password_max_age"], default=0),
            "pw_reuse_prevent": dict(type="int", aliases=["password_reuse_prevent", "prevent_reuse"], default=0),
            "pw_expire": dict(type="bool", aliases=["password_expire", "expire"], default=False),
        },
        supports_check_mode=True,
    )

    resource = IAMConnection(module)
    policy = resource.connection.AccountPasswordPolicy()

    state = module.params.get("state")

    if state == "present":
        (changed, new_policy, update_result) = resource.update_password_policy(module, policy)
        module.exit_json(changed=changed, task_status={"IAM": update_result}, policy=new_policy)

    if state == "absent":
        delete_result = resource.delete_password_policy(policy)
        module.exit_json(changed=True, task_status={"IAM": delete_result})


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_carrier_gateway
version_added: 6.0.0
short_description: Manage an AWS VPC Carrier gateway
description:
  - Manage an AWS VPC Carrier gateway.
author:
  - "Marco Braga (@mtulio)"
options:
  vpc_id:
    description:
      - The VPC ID for the VPC in which to manage the Carrier Gateway.
    required: true
    type: str
  carrier_gateway_id:
    description:
      - The Carrier Gateway ID to manage the Carrier Gateway.
    required: false
    type: str
  state:
    description:
      - Create or terminate the Carrier Gateway.
    default: present
    choices: [ 'present', 'absent' ]
    type: str
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

# Ensure that the VPC has an Carrier Gateway.
# The Carrier Gateway ID can be accessed via {{cagw.carrier_gateway_id}} for use in setting up Route tables etc.
- name: Create Carrier gateway
  community.aws.ec2_carrier_gateway:
    vpc_id: vpc-abcdefgh
    state: present
  register: cagw

- name: Create Carrier gateway with tags
  community.aws.ec2_carrier_gateway:
    vpc_id: vpc-abcdefgh
    state: present
    tags:
      Tag1: tag1
      Tag2: tag2
  register: cagw

- name: Delete Carrier gateway
  community.aws.ec2_carrier_gateway:
    vpc_id: vpc-abcdefgh
    carrier_gateway_id: "cagw-123"
    state: absent
  register: vpc_cagw_delete
"""

RETURN = r"""
changed:
  description: If any changes have been made to the Carrier Gateway.
  type: bool
  returned: always
  sample:
    changed: false
carrier_gateway_id:
  description: The unique identifier for the Carrier Gateway.
  type: str
  returned: I(state=present)
  sample:
    carrier_gateway_id: "cagw-XXXXXXXX"
tags:
  description: The tags associated the Carrier Gateway.
  type: dict
  returned: I(state=present)
  sample:
    tags:
      "Ansible": "Test"
vpc_id:
  description: The VPC ID associated with the Carrier Gateway.
  type: str
  returned: I(state=present)
  sample:
    vpc_id: "vpc-XXXXXXXX"
"""

try:
    import botocore
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_message
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import ensure_ec2_tags
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


@AWSRetry.jittered_backoff(retries=10, delay=10)
def describe_cagws_with_backoff(connection, **params):
    paginator = connection.get_paginator("describe_carrier_gateways")
    return paginator.paginate(**params).build_full_result()["CarrierGateways"]


class AnsibleEc2Cagw:
    def __init__(self, module, results):
        self._module = module
        self._results = results
        self._connection = self._module.client("ec2", retry_decorator=AWSRetry.jittered_backoff())
        self._check_mode = self._module.check_mode

    def process(self):
        vpc_id = self._module.params.get("vpc_id")
        state = self._module.params.get("state", "present")
        tags = self._module.params.get("tags")
        purge_tags = self._module.params.get("purge_tags")

        if state == "present":
            self.ensure_cagw_present(vpc_id, tags, purge_tags)
        elif state == "absent":
            self.ensure_cagw_absent(vpc_id)

    def get_matching_cagw(self, vpc_id, carrier_gateway_id=None):
        """
        Returns the carrier gateway found.
            Parameters:
                vpc_id (str): VPC ID
                carrier_gateway_id (str): Carrier Gateway ID, if specified
            Returns:
                cagw (dict): dict of cagw found, None if none found
        """
        filters = ansible_dict_to_boto3_filter_list({"vpc-id": vpc_id})
        try:
            if not carrier_gateway_id:
                cagws = describe_cagws_with_backoff(
                    self._connection,
                    Filters=filters,
                )
            else:
                cagws = describe_cagws_with_backoff(
                    self._connection,
                    CarrierGatewayIds=[carrier_gateway_id],
                )
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            self._module.fail_json_aws(e)

        cagw = None
        if len(cagws) > 1:
            self._module.fail_json(msg=f"EC2 returned more than one Carrier Gateway for VPC {vpc_id}, aborting")
        elif cagws:
            cagw = camel_dict_to_snake_dict(cagws[0])

        return cagw

    @staticmethod
    def get_cagw_info(cagw, vpc_id):
        return {
            "carrier_gateway_id": cagw["carrier_gateway_id"],
            "tags": boto3_tag_list_to_ansible_dict(cagw["tags"]),
            "vpc_id": vpc_id,
        }

    def ensure_cagw_absent(self, vpc_id):
        cagw = self.get_matching_cagw(vpc_id)
        if cagw is None:
            return self._results

        if self._check_mode:
            self._results["changed"] = True
            return self._results

        try:
            self._results["changed"] = True
            self._connection.delete_carrier_gateway(
                aws_retry=True,
                CarrierGatewayId=cagw["carrier_gateway_id"],
            )
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            self._module.fail_json_aws(e, msg="Unable to delete Carrier Gateway")

        return self._results

    def ensure_cagw_present(self, vpc_id, tags, purge_tags):
        cagw = self.get_matching_cagw(vpc_id)

        if cagw is None:
            if self._check_mode:
                self._results["changed"] = True
                self._results["carrier_gateway_id"] = None
                return self._results

            try:
                response = self._connection.create_carrier_gateway(VpcId=vpc_id, aws_retry=True)
                cagw = camel_dict_to_snake_dict(response["CarrierGateway"])
                self._results["changed"] = True
            except is_boto3_error_message("You must be opted into a wavelength zone to create a carrier gateway.") as e:
                self._module.fail_json(msg="You must be opted into a wavelength zone to create a carrier gateway")
            except botocore.exceptions.WaiterError as e:
                self._module.fail_json_aws(e, msg="No Carrier Gateway exists.")
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                self._module.fail_json_aws(e, msg="Unable to create Carrier Gateway")

        # Modify tags
        self._results["changed"] |= ensure_ec2_tags(
            self._connection,
            self._module,
            cagw["carrier_gateway_id"],
            resource_type="carrier-gateway",
            tags=tags,
            purge_tags=purge_tags,
            retry_codes="InvalidCarrierGatewayID.NotFound",
        )

        # Update cagw
        cagw = self.get_matching_cagw(vpc_id, carrier_gateway_id=cagw["carrier_gateway_id"])
        cagw_info = self.get_cagw_info(cagw, vpc_id)
        self._results.update(cagw_info)

        return self._results


def main():
    argument_spec = dict(
        carrier_gateway_id=dict(required=False),
        vpc_id=dict(required=True),
        state=dict(default="present", choices=["present", "absent"]),
        tags=dict(required=False, type="dict", aliases=["resource_tags"]),
        purge_tags=dict(default=True, type="bool"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        required_one_of=[["vpc_id", "carrier_gateway_id"]],
        supports_check_mode=True,
    )
    results = dict(
        changed=False,
    )
    cagw_manager = AnsibleEc2Cagw(module=module, results=results)
    cagw_manager.process()

    module.exit_json(**results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# coding: utf-8 -*-

# Copyright (c) Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: get_certificate
author: "John Westcott IV (@john-westcott-iv)"
short_description: Get a certificate from a host:port
description:
    - Makes a secure connection and returns information about the presented certificate
    - The module uses the cryptography Python library.
    - Support SNI (L(Server Name Indication,https://en.wikipedia.org/wiki/Server_Name_Indication)) only with python >= 2.7.
extends_documentation_fragment:
    - community.crypto.attributes
attributes:
    check_mode:
        support: none
        details:
            - This action does not modify state.
    diff_mode:
        support: N/A
        details:
            - This action does not modify state.
options:
    host:
      description:
        - The host to get the cert for (IP is fine)
      type: str
      required: true
    ca_cert:
      description:
        - A PEM file containing one or more root certificates; if present, the cert will be validated against these root certs.
        - Note that this only validates the certificate is signed by the chain; not that the cert is valid for the host presenting it.
      type: path
    port:
      description:
        - The port to connect to
      type: int
      required: true
    server_name:
      description:
       - Server name used for SNI (L(Server Name Indication,https://en.wikipedia.org/wiki/Server_Name_Indication)) when hostname
         is an IP or is different from server name.
      type: str
      version_added: 1.4.0
    proxy_host:
      description:
        - Proxy host used when get a certificate.
      type: str
    proxy_port:
      description:
        - Proxy port used when get a certificate.
      type: int
      default: 8080
    starttls:
      description:
        - Requests a secure connection for protocols which require clients to initiate encryption.
        - Only available for V(mysql) currently.
      type: str
      choices:
        - mysql
      version_added: 1.9.0
    timeout:
      description:
        - The timeout in seconds
      type: int
      default: 10
    select_crypto_backend:
      description:
        - Determines which crypto backend to use.
        - The default choice is V(auto), which tries to use C(cryptography) if available.
        - If set to V(cryptography), will try to use the L(cryptography,https://cryptography.io/) library.
      type: str
      default: auto
      choices: [ auto, cryptography ]
    ciphers:
      description:
        - SSL/TLS Ciphers to use for the request.
        - 'When a list is provided, all ciphers are joined in order with V(:).'
        - See the L(OpenSSL Cipher List Format,https://www.openssl.org/docs/manmaster/man1/openssl-ciphers.html#CIPHER-LIST-FORMAT)
          for more details.
        - The available ciphers is dependent on the Python and OpenSSL/LibreSSL versions.
      type: list
      elements: str
      version_added: 2.11.0
    asn1_base64:
      description:
        - Whether to encode the ASN.1 values in the RV(extensions) return value with Base64 or not.
        - The documentation claimed for a long time that the values are Base64 encoded, but they
          never were. For compatibility this option is set to V(false).
        - The default value V(false) is B(deprecated) and will change to V(true) in community.crypto 3.0.0.
      type: bool
      version_added: 2.12.0

notes:
    - When using ca_cert on OS X it has been reported that in some conditions the validate will always succeed.

requirements:
    - "python >= 2.7 when using O(proxy_host)"
    - "cryptography >= 1.6"

seealso:
    - plugin: community.crypto.to_serial
      plugin_type: filter
'''

RETURN = '''
cert:
    description: The certificate retrieved from the port
    returned: success
    type: str
expired:
    description: Boolean indicating if the cert is expired
    returned: success
    type: bool
extensions:
    description: Extensions applied to the cert
    returned: success
    type: list
    elements: dict
    contains:
        critical:
            returned: success
            type: bool
            description: Whether the extension is critical.
        asn1_data:
            returned: success
            type: str
            description:
              - The ASN.1 content of the extension.
              - If O(asn1_base64=true) this will be Base64 encoded, otherwise the raw
                binary value will be returned.
              - Please note that the raw binary value might not survive JSON serialization
                to the Ansible controller, and also might cause failures when displaying it.
                See U(https://github.com/ansible/ansible/issues/80258) for more information.
              - B(Note) that depending on the C(cryptography) version used, it is
                not possible to extract the ASN.1 content of the extension, but only
                to provide the re-encoded content of the extension in case it was
                parsed by C(cryptography). This should usually result in exactly the
                same value, except if the original extension value was malformed.
        name:
            returned: success
            type: str
            description: The extension's name.
issuer:
    description: Information about the issuer of the cert.
    returned: success
    type: dict
not_after:
    description: Expiration date of the cert.
    returned: success
    type: str
not_before:
    description: Issue date of the cert.
    returned: success
    type: str
serial_number:
    description:
        - The serial number of the cert.
        - This return value is an B(integer). If you need the serial numbers as a colon-separated hex string,
          such as C(11:22:33), you need to convert it to that form with P(community.crypto.to_serial#filter).
    returned: success
    type: int
signature_algorithm:
    description: The algorithm used to sign the cert.
    returned: success
    type: str
subject:
    description: Information about the subject of the cert (C(OU), C(CN), etc).
    returned: success
    type: dict
version:
    description: The version number of the certificate.
    returned: success
    type: str
'''

EXAMPLES = '''
- name: Get the cert from an RDP port
  community.crypto.get_certificate:
    host: "1.2.3.4"
    port: 3389
  delegate_to: localhost
  run_once: true
  register: cert

- name: Get a cert from an https port
  community.crypto.get_certificate:
    host: "www.google.com"
    port: 443
  delegate_to: localhost
  run_once: true
  register: cert

- name: How many days until cert expires
  ansible.builtin.debug:
    msg: "cert expires in: {{ expire_days }} days."
  vars:
    expire_days: "{{ (( cert.not_after | to_datetime('%Y%m%d%H%M%SZ')) - (ansible_date_time.iso8601 | to_datetime('%Y-%m-%dT%H:%M:%SZ')) ).days }}"
'''

import atexit
import base64
import datetime
import traceback

from os.path import isfile
from socket import create_connection, setdefaulttimeout, socket
from ssl import get_server_certificate, DER_cert_to_PEM_cert, CERT_NONE, CERT_REQUIRED

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils.common.text.converters import to_bytes

from ansible_collections.community.crypto.plugins.module_utils.version import LooseVersion

from ansible_collections.community.crypto.plugins.module_utils.crypto.cryptography_support import (
    cryptography_oid_to_name,
    cryptography_get_extensions_from_cert,
)

MINIMAL_CRYPTOGRAPHY_VERSION = '1.6'

CREATE_DEFAULT_CONTEXT_IMP_ERR = None
try:
    from ssl import create_default_context
except ImportError:
    CREATE_DEFAULT_CONTEXT_IMP_ERR = traceback.format_exc()
    HAS_CREATE_DEFAULT_CONTEXT = False
else:
    HAS_CREATE_DEFAULT_CONTEXT = True

CRYPTOGRAPHY_IMP_ERR = None
try:
    import cryptography
    import cryptography.exceptions
    import cryptography.x509
    from cryptography.hazmat.backends import default_backend as cryptography_backend
    CRYPTOGRAPHY_VERSION = LooseVersion(cryptography.__version__)
except ImportError:
    CRYPTOGRAPHY_IMP_ERR = traceback.format_exc()
    CRYPTOGRAPHY_FOUND = False
else:
    CRYPTOGRAPHY_FOUND = True


def send_starttls_packet(sock, server_type):
    if server_type == 'mysql':
        ssl_request_packet = (
            b'\x20\x00\x00\x01\x85\xae\x7f\x00' +
            b'\x00\x00\x00\x01\x21\x00\x00\x00' +
            b'\x00\x00\x00\x00\x00\x00\x00\x00' +
            b'\x00\x00\x00\x00\x00\x00\x00\x00' +
            b'\x00\x00\x00\x00'
        )

        sock.recv(8192)  # discard initial handshake from server for this naive implementation
        sock.send(ssl_request_packet)


def main():
    module = AnsibleModule(
        argument_spec=dict(
            ca_cert=dict(type='path'),
            host=dict(type='str', required=True),
            port=dict(type='int', required=True),
            proxy_host=dict(type='str'),
            proxy_port=dict(type='int', default=8080),
            server_name=dict(type='str'),
            timeout=dict(type='int', default=10),
            select_crypto_backend=dict(type='str', choices=['auto', 'cryptography'], default='auto'),
            starttls=dict(type='str', choices=['mysql']),
            ciphers=dict(type='list', elements='str'),
            asn1_base64=dict(type='bool'),
        ),
    )

    ca_cert = module.params.get('ca_cert')
    host = module.params.get('host')
    port = module.params.get('port')
    proxy_host = module.params.get('proxy_host')
    proxy_port = module.params.get('proxy_port')
    timeout = module.params.get('timeout')
    server_name = module.params.get('server_name')
    start_tls_server_type = module.params.get('starttls')
    ciphers = module.params.get('ciphers')
    asn1_base64 = module.params['asn1_base64']
    if asn1_base64 is None:
        module.deprecate(
            'The default value `false` for asn1_base64 is deprecated and will change to `true` in '
            'community.crypto 3.0.0. If you need this value, it is best to set the value explicitly '
            'and adjust your roles/playbooks to use `asn1_base64=true` as soon as possible',
            version='3.0.0',
            collection_name='community.crypto',
        )
        asn1_base64 = False

    backend = module.params.get('select_crypto_backend')
    if backend == 'auto':
        # Detection what is possible
        can_use_cryptography = CRYPTOGRAPHY_FOUND and CRYPTOGRAPHY_VERSION >= LooseVersion(MINIMAL_CRYPTOGRAPHY_VERSION)

        # Try cryptography
        if can_use_cryptography:
            backend = 'cryptography'

        # Success?
        if backend == 'auto':
            module.fail_json(msg=("Cannot detect the required Python library "
                                  "cryptography (>= {0})").format(MINIMAL_CRYPTOGRAPHY_VERSION))

    if backend == 'cryptography':
        if not CRYPTOGRAPHY_FOUND:
            module.fail_json(msg=missing_required_lib('cryptography >= {0}'.format(MINIMAL_CRYPTOGRAPHY_VERSION)),
                             exception=CRYPTOGRAPHY_IMP_ERR)

    result = dict(
        changed=False,
    )

    if timeout:
        setdefaulttimeout(timeout)

    if ca_cert:
        if not isfile(ca_cert):
            module.fail_json(msg="ca_cert file does not exist")

    if not HAS_CREATE_DEFAULT_CONTEXT:
        # Python < 2.7.9
        if proxy_host:
            module.fail_json(msg='To use proxy_host, you must run the get_certificate module with Python 2.7 or newer.',
                             exception=CREATE_DEFAULT_CONTEXT_IMP_ERR)
        if ciphers is not None:
            module.fail_json(msg='To use ciphers, you must run the get_certificate module with Python 2.7 or newer.',
                             exception=CREATE_DEFAULT_CONTEXT_IMP_ERR)
        try:
            # Note: get_server_certificate does not support SNI!
            cert = get_server_certificate((host, port), ca_certs=ca_cert)
        except Exception as e:
            module.fail_json(msg="Failed to get cert from {0}:{1}, error: {2}".format(host, port, e))
    else:
        # Python >= 2.7.9
        try:
            if proxy_host:
                connect = "CONNECT %s:%s HTTP/1.0\r\n\r\n" % (host, port)
                sock = socket()
                atexit.register(sock.close)
                sock.connect((proxy_host, proxy_port))
                sock.send(connect.encode())
                sock.recv(8192)
            else:
                sock = create_connection((host, port))
                atexit.register(sock.close)

            if ca_cert:
                ctx = create_default_context(cafile=ca_cert)
                ctx.check_hostname = False
                ctx.verify_mode = CERT_REQUIRED
            else:
                ctx = create_default_context()
                ctx.check_hostname = False
                ctx.verify_mode = CERT_NONE

            if start_tls_server_type is not None:
                send_starttls_packet(sock, start_tls_server_type)

            if ciphers is not None:
                ciphers_joined = ":".join(ciphers)
                ctx.set_ciphers(ciphers_joined)

            cert = ctx.wrap_socket(sock, server_hostname=server_name or host).getpeercert(True)
            cert = DER_cert_to_PEM_cert(cert)
        except Exception as e:
            if proxy_host:
                module.fail_json(msg="Failed to get cert via proxy {0}:{1} from {2}:{3}, error: {4}".format(
                    proxy_host, proxy_port, host, port, e))
            else:
                module.fail_json(msg="Failed to get cert from {0}:{1}, error: {2}".format(host, port, e))

    result['cert'] = cert

    if backend == 'cryptography':
        x509 = cryptography.x509.load_pem_x509_certificate(to_bytes(cert), cryptography_backend())
        result['subject'] = {}
        for attribute in x509.subject:
            result['subject'][cryptography_oid_to_name(attribute.oid, short=True)] = attribute.value

        result['expired'] = x509.not_valid_after < datetime.datetime.utcnow()

        result['extensions'] = []
        for dotted_number, entry in cryptography_get_extensions_from_cert(x509).items():
            oid = cryptography.x509.oid.ObjectIdentifier(dotted_number)
            ext = {
                'critical': entry['critical'],
                'asn1_data': entry['value'],
                'name': cryptography_oid_to_name(oid, short=True),
            }
            if not asn1_base64:
                ext['asn1_data'] = base64.b64decode(ext['asn1_data'])
            result['extensions'].append(ext)

        result['issuer'] = {}
        for attribute in x509.issuer:
            result['issuer'][cryptography_oid_to_name(attribute.oid, short=True)] = attribute.value

        result['not_after'] = x509.not_valid_after.strftime('%Y%m%d%H%M%SZ')
        result['not_before'] = x509.not_valid_before.strftime('%Y%m%d%H%M%SZ')

        result['serial_number'] = x509.serial_number
        result['signature_algorithm'] = cryptography_oid_to_name(x509.signature_algorithm_oid)

        # We need the -1 offset to get the same values as pyOpenSSL
        if x509.version == cryptography.x509.Version.v1:
            result['version'] = 1 - 1
        elif x509.version == cryptography.x509.Version.v3:
            result['version'] = 3 - 1
        else:
            result['version'] = "unknown"

    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2021 Ross Bender (@l3ender)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_appgateway_info
version_added: "1.10.0"
short_description: Retrieve Application Gateway instance facts
description:
    - Get facts for a Application Gateway instance.
options:
    name:
        description:
            - Only show results for a specific application gateway.
        type: str
    resource_group:
        description:
            - Limit results by resource group.
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Ross Bender (@l3ender)
'''

EXAMPLES = '''
- name: Get facts for application gateway by name.
  azure_rm_appgateway_info:
    name: MyAppgw
    resource_group: MyResourceGroup

- name: Get facts for application gateways in resource group.
  azure_rm_appgateway_info:
    resource_group: MyResourceGroup

- name: Get facts for all application gateways.
  azure_rm_appgateway_info:
'''

RETURN = '''
gateways:
    description:
        - A list of dictionaries containing facts for an application gateway.
    returned: always
    type: list
    elements: dict
    contains:
        id:
            description:
                - Application gateway resource ID.
            returned: always
            type: str
            sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/applicationGateways/myAppGw
        name:
            description:
                - Name of application gateway.
            returned: always
            type: str
            sample: myAppGw
        resource_group:
            description:
                - Name of resource group.
            returned: always
            type: str
            sample: myResourceGroup
        location:
            description:
                - Location of application gateway.
            returned: always
            type: str
            sample: centralus
        operational_state:
            description:
                - Operating state of application gateway.
            returned: always
            type: str
            sample: Running
        provisioning_state:
            description:
                - Provisioning state of application gateway.
            returned: always
            type: str
            sample: Succeeded
        ssl_policy:
            description:
                - SSL policy of the application gateway.
            returned: always
            type: complex
            version_added: "1.11.0"
            contains:
                policy_type:
                    description:
                        - The type of SSL policy.
                    returned: always
                    type: str
                    sample: predefined
                policy_name:
                    description:
                        - The name of the SSL policy.
                    returned: always
                    type: str
                    sample: ssl_policy20170401_s
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
from ansible.module_utils.common.dict_transformations import _camel_to_snake

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.mgmt.core.tools import parse_resource_id
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMApplicationGatewayInfo(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type='str'),
            resource_group=dict(type='str'),
        )

        self.results = dict(
            changed=False,
        )

        self.name = None
        self.resource_group = None

        super(AzureRMApplicationGatewayInfo, self).__init__(self.module_arg_spec,
                                                            supports_check_mode=True,
                                                            supports_tags=False,
                                                            facts_module=True)

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name is not None:
            self.results["gateways"] = self.get()
        elif self.resource_group is not None:
            self.results["gateways"] = self.list_by_rg()
        else:
            self.results["gateways"] = self.list_all()

        return self.results

    def get(self):
        response = None
        results = []
        try:
            response = self.network_client.application_gateways.get(resource_group_name=self.resource_group, application_gateway_name=self.name)
        except ResourceNotFoundError:
            pass

        if response is not None:
            results.append(self.format_response(response))

        return results

    def list_by_rg(self):
        response = None
        results = []
        try:
            response = self.network_client.application_gateways.list(resource_group_name=self.resource_group)
        except Exception as exc:
            request_id = exc.request_id if exc.request_id else ''
            self.fail("Error listing application gateways in resource groups {0}: {1} - {2}".format(self.resource_group, request_id, str(exc)))

        for item in response:
            results.append(self.format_response(item))

        return results

    def list_all(self):
        response = None
        results = []
        try:
            response = self.network_client.application_gateways.list_all()
        except Exception as exc:
            request_id = exc.request_id if exc.request_id else ''
            self.fail("Error listing all application gateways: {0} - {1}".format(request_id, str(exc)))

        for item in response:
            results.append(self.format_response(item))

        return results

    def format_response(self, appgw):
        d = appgw.as_dict()
        id = d.get("id")
        id_dict = parse_resource_id(id)
        d = {
            "id": id,
            "name": d.get("name"),
            "resource_group": id_dict.get('resource_group', self.resource_group),
            "location": d.get("location"),
            "operational_state": d.get("operational_state"),
            "provisioning_state": d.get("provisioning_state"),
            "ssl_policy": None if d.get("ssl_policy") is None else {
                "policy_type": _camel_to_snake(d.get("ssl_policy").get("policy_type", None)),
                "policy_name": self.ssl_policy_name(d.get("ssl_policy").get("policy_name", None)),
            },
        }
        return d

    def ssl_policy_name(self, policy_name):
        if policy_name == "AppGwSslPolicy20150501":
            return "ssl_policy20150501"
        elif policy_name == "AppGwSslPolicy20170401":
            return "ssl_policy20170401"
        elif policy_name == "AppGwSslPolicy20170401S":
            return "ssl_policy20170401_s"
        return None


def main():
    AzureRMApplicationGatewayInfo()


if __name__ == '__main__':
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2022 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The iosxr_ping config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.iosxr import run_commands
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.rm_templates.ping import (
    PingTemplate,
)


class Ping:
    """
    The iosxr_ping config class
    """

    def __init__(self, module):
        self.module = module
        self.result = {}

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        self.generate_command()
        res = self.run_command()
        return self.process_result(res)

    def build_ping(self, params):
        tmplt = PingTemplate()
        params = utils.remove_empties(params)
        cmd = tmplt.render(params, "rate", False)
        return cmd

    def validate_results(self, module, loss, results):
        """
        This function is used to validate whether the ping results were unexpected per "state" param.
        """
        state = module.params["state"]
        if state == "present" and loss == 100:
            module.fail_json(msg="Ping failed unexpectedly", **results)
        elif state == "absent" and loss < 100:
            module.fail_json(msg="Ping succeeded unexpectedly", **results)

    def generate_command(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """
        warnings = list()
        if warnings:
            self.result["warnings"] = warnings
        self.result["commands"] = self.build_ping(self.module.params)

    def run_command(self):
        ping_results = run_commands(
            self.module,
            commands=self.result["commands"],
        )
        return ping_results

    def process_result(self, ping_results):
        """
        Function used to parse the statistical information from the ping response.
        Example: "Success rate is 100 percent (5/5), round-trip min/avg/max = 1/2/8 ms"
        Returns the percent of packet loss, received packets, transmitted packets, and RTT data.
        """

        if isinstance(ping_results, list):
            ping_results = ping_results[0]

        ping_data = PingTemplate(lines=ping_results.splitlines())
        obj = list(ping_data.parse().values())

        self.result["packet_loss"] = obj[0].get("loss_percentage")
        self.result["packets_rx"] = obj[0].get("rx")
        self.result["packets_tx"] = obj[0].get("tx")
        self.result["rtt"] = obj[0].get("rtt")
        loss = obj[0].get("loss")
        self.validate_results(self.module, loss, self.result)
        return self.result



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: hotpatch_install
short_description: Resource module for Hotpatch Install
description:
- Manage operation create of the resource Hotpatch Install.
- >
   Triggers hot patch installation on the Cisco ISE node. A task ID is returned which can be used to monitor the progress of the hot patch installation
   process. As hot patch installation triggers the Cisco ISE to restart, the task API becomes unavailable for a certain period of time.
version_added: '2.1.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  hotpatchName:
    description: Hotpatch Install's hotpatchName.
    type: str
  repositoryName:
    description: Hotpatch Install's repositoryName.
    type: str
requirements:
- ciscoisesdk >= 2.1.1
- python >= 3.5
seealso:
- name: Cisco ISE documentation for Patching
  description: Complete reference of the Patching API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!patch-and-hot-patch-openapi
notes:
  - SDK Method used are
    patching.Patching.install_hotpatch,

  - Paths used are
    post /api/v1/hotpatch/install,

"""

EXAMPLES = r"""
- name: Create
  cisco.ise.hotpatch_install:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    hotpatchName: string
    repositoryName: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "id": "string",
        "message": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: personas_register_node
short_description: Register a node to the primary
description:
- Register a node to the primary
version_added: '2.4.0'
author: Rafael Campos (@racampos)
options:
  primary_ip:
    description:
    - The IP address of the primary node.
    type: str
  primary_username:
    description:
    - The username for the primary node.
    type: str
  primary_password:
    description:
    - The password for the primary node.
    type: str
  fqdn:
    description:
    - The fully qualified domain name of the node.
    type: str
  username:
    description:
    - The username to log into the node.
    type: str
  password:
    description:
    - The password to log into the node.
    type: str
  roles:
    description:
    - "The roles to be fulfilled by this node. Possible roles are PrimaryAdmin, SecondaryAdmin, \
      PrimaryMonitoring, SecondaryMonitoring, PrimaryDedicatedMonitoring, SecondaryDedicatedMonitoring, Standalone"
    type: list
    elements: str
  services:
    description:
    - The services this node will run. Possible services are Session, Profiler, TC-NAC, SXP, DeviceAdmin, PassiveIdentity, pxGrid, pxGridCloud
    type: list
    elements: str
  ise_verify:
    description:
    - Whether or not to verify the identity of the node.
    type: bool
  ise_version:
    description:
    - The version of the ISE node.
    type: str
  ise_wait_on_rate_limit:
    description:
    - Whether or not to wait on rate limit
    type: bool
requirements:
- requests >= 2.25.1
- python >= 3.5
seealso:
# Reference by module name
- module: cisco.ise.plugins.modules.personas_register_node
notes:
    - "Does not support C(check_mode)"
"""

EXAMPLES = r"""
- name: Register the secondary node and PSN nodes to the cluster
  cisco.ise.personas_register_node:
    primary_ip: 10.1.1.1
    primary_username: admin
    primary_password: Cisco123
    fqdn: "{{ item.fqdn }}"
    username: admin
    password: cisco123
    roles: "{{ item.roles }}"
    services: "{{ item.services }}"
  loop:
    - fqdn: ise-pan-server-2.example.com
      roles:
        - SecondaryAdmin
        - SecondaryMonitoring
      services: []
    - fqdn: ise-psn-server-1.example.com
      roles: []
      services:
        - Session
        - Profiler
    - fqdn: ise-psn-server-2.example.com
      roles: []
      services:
        - Session
        - Profiler
"""

RETURN = r"""
ise_response:
  description: A string stating that the node was successfully registered
  returned: always
  type: str
  sample: Node ise-pan-server-2 updated successfully
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_trusted_client
short_description: Manages trusted-client objects on Checkpoint over Web Services API
description:
  - Manages trusted-client objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
version_added: "2.1.0"
author: "Or Soffer (@chkp-orso)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  ip_address:
    description:
      - IPv4 or IPv6 address. If both addresses are required use ipv4-address and ipv6-address fields explicitly.
    type: str
  ipv4_address:
    description:
      - IPv4 address.
    type: str
  ipv6_address:
    description:
      - IPv6 address.
    type: str
  domains_assignment:
    description:
      - Domains to be added to this profile. Use domain name only. See example below, "add-trusted-client (with domain)".
    type: list
    elements: str
  ip_address_first:
    description:
      - First IP address in the range. If both IPv4 and IPv6 address ranges are required, use the ipv4-address-first and the ipv6-address-first fields instead.
    type: str
  ipv4_address_first:
    description:
      - First IPv4 address in the range.
    type: str
  ipv6_address_first:
    description:
      - First IPv6 address in the range.
    type: str
  ip_address_last:
    description:
      - Last IP address in the range. If both IPv4 and IPv6 address ranges are required, use the ipv4-address-first and the ipv6-address-first fields instead.
    type: str
  ipv4_address_last:
    description:
      - Last IPv4 address in the range.
    type: str
  ipv6_address_last:
    description:
      - Last IPv6 address in the range.
    type: str
  mask_length:
    description:
      - IPv4 or IPv6 mask length. If both masks are required use mask-length4 and mask-length6 fields explicitly.
    type: int
  mask_length4:
    description:
      - IPv4 mask length.
    type: int
  mask_length6:
    description:
      - IPv6 mask length.
    type: int
  multi_domain_server_trusted_client:
    description:
      - Let this trusted client connect to all Multi-Domain Servers in the deployment.
    type: bool
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  type:
    description:
      - Trusted client type.
    type: str
    choices: ['any', 'domain', 'ipv4 address', 'ipv4 address range', 'ipv4 netmask', 'ipv6 address', 'ipv6 address range', 'ipv6 netmask', 'name',
             'wild cards (ip only)']
  wild_card:
    description:
      - IP wild card (e.g. 192.0.2.*).
    type: str
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-trusted-client
  cp_mgmt_trusted_client:
    name: my client
    state: present
    type: ANY

- name: set-trusted-client
  cp_mgmt_trusted_client:
    ip_address: 192.0.2.1
    mask_length: '24'
    name: my client
    state: present
    type: NETMASK

- name: delete-trusted-client
  cp_mgmt_trusted_client:
    name: my client
    state: absent
"""

RETURN = """
cp_mgmt_trusted_client:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_objects,
    api_call,
)


def main():
    argument_spec = dict(
        name=dict(type="str", required=True),
        ip_address=dict(type="str"),
        ipv4_address=dict(type="str"),
        ipv6_address=dict(type="str"),
        domains_assignment=dict(type="list", elements="str"),
        ip_address_first=dict(type="str"),
        ipv4_address_first=dict(type="str"),
        ipv6_address_first=dict(type="str"),
        ip_address_last=dict(type="str"),
        ipv4_address_last=dict(type="str"),
        ipv6_address_last=dict(type="str"),
        mask_length=dict(type="int"),
        mask_length4=dict(type="int"),
        mask_length6=dict(type="int"),
        multi_domain_server_trusted_client=dict(type="bool"),
        tags=dict(type="list", elements="str"),
        type=dict(
            type="str",
            choices=[
                "any",
                "domain",
                "ipv4 address",
                "ipv4 address range",
                "ipv4 netmask",
                "ipv6 address",
                "ipv6 address range",
                "ipv6 netmask",
                "name",
                "wild cards (ip only)",
            ],
        ),
        wild_card=dict(type="str"),
        color=dict(
            type="str",
            choices=[
                "aquamarine",
                "black",
                "blue",
                "crete blue",
                "burlywood",
                "cyan",
                "dark green",
                "khaki",
                "orchid",
                "dark orange",
                "dark sea green",
                "pink",
                "turquoise",
                "dark blue",
                "firebrick",
                "brown",
                "forest green",
                "gold",
                "dark gold",
                "gray",
                "dark gray",
                "light green",
                "lemon chiffon",
                "coral",
                "sea green",
                "sky blue",
                "magenta",
                "purple",
                "slate blue",
                "violet red",
                "navy blue",
                "olive",
                "orange",
                "red",
                "sienna",
                "yellow",
            ],
        ),
        comments=dict(type="str"),
        details_level=dict(type="str", choices=["uid", "standard", "full"]),
        ignore_warnings=dict(type="bool"),
        ignore_errors=dict(type="bool"),
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(
        argument_spec=argument_spec, supports_check_mode=True
    )
    api_call_object = "trusted-client"

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import re

from ansible.module_utils.common.text.converters import to_text


_ASCII_PRINTABLE_MATCHER = re.compile(r'^[\x20-\x7e]*$')


def is_ascii_label(domain):
    '''
    Check whether domain name has only ASCII labels.
    '''
    return _ASCII_PRINTABLE_MATCHER.match(domain) is not None


class InvalidDomainName(Exception):
    '''
    The provided domain name is not valid.
    '''
    pass


def split_into_labels(domain):
    '''
    Split domain name to a list of labels. Start with the top-most label.

    Returns a list of labels and a tail, which is either ``''`` or ``'.'``.
    Raises ``InvalidDomainName`` if the domain name is not valid.
    '''
    result = []
    index = len(domain)
    tail = ''
    if domain.endswith('.'):
        index -= 1
        tail = '.'
    if index > 0:
        while index >= 0:
            next_index = domain.rfind('.', 0, index)
            label = domain[next_index + 1:index]
            if label == '' or label[0] == '-' or label[-1] == '-' or len(label) > 63:
                raise InvalidDomainName(domain)
            result.append(label)
            index = next_index
    return result, tail


def join_labels(labels, tail=''):
    '''
    Combines the result of split_into_labels() back into a domain name.
    '''
    return '.'.join(reversed(labels)) + tail


def normalize_label(label):
    '''
    Normalize a domain label. Returns a lower-case ASCII label.

    If a ulabel is provided, it is converted to an alabel.
    '''
    if not is_ascii_label(label):
        # Convert ulabel to alabel
        label = to_text(b'xn--' + to_text(label).encode('punycode'))
    # Always convert to lower-case
    return label.lower()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: dataconnect_settings_password_expiry
short_description: Resource module for Dataconnect Settings Password Expiry
description:
- Manage operation update of the resource Dataconnect Settings Password Expiry.
- This API updates the number of days of Dataconnect password expiry.
version_added: '2.8.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  passwordExpiresInDays:
    description: Dataconnect Settings Password Expiry's passwordExpiresInDays.
    type: int
requirements:
- ciscoisesdk >= 2.1.1
- python >= 3.5
notes:
  - SDK Method used are
    dataconnect_services.DataconnectServices.update_dataconnect_password_expiry,

  - Paths used are
    put /api/v1/mnt/data-connect/settings/password/expiry,

"""

EXAMPLES = r"""
- name: Update all
  cisco.ise.dataconnect_settings_password_expiry:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    passwordExpiresInDays: 0

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "success": {
        "message": "string"
      },
      "version": "string"
    }
"""
